@@ -88,7 +88,7 @@ static void locate_unmanaged_files(
     for(i=0; i<argc; i++){
       file_canonical_name(argv[i], &name, 0);
       zName = blob_str(&name);
-      isDir = file_wd_isdir(zName);
+      isDir = file_isdir(zName, RepoFILE);
       if( isDir==1 ){
         vfile_scan(&name, nRoot-1, scanFlags, pIgnore, 0);
       }else if( isDir==0 ){
@@ -211,7 +211,7 @@ static void status_report(
     int isNew = isManaged && !db_column_int(&q, 5);
     int isRenamed = db_column_int(&q, 6);
     char *zFullName = mprintf("%s%s", g.zLocalRoot, zPathname);
-    int isMissing = !file_wd_isfile_or_link(zFullName);
+    int isMissing = !file_isfile_or_link(zFullName);
 
     /* Determine the file change classification, if any. */
     if( isDeleted ){
@@ -256,7 +256,7 @@ static void status_report(
       zClass = "UNEXEC";
     }else if( (flags & C_META) && isChnged==9 ){
       zClass = "UNLINK";
-    }else if( (flags & C_CONFLICT) && isChnged && !file_wd_islink(zFullName)
+    }else if( (flags & C_CONFLICT) && isChnged && !file_islink(zFullName)
            && file_contains_merge_marker(zFullName) ){
       zClass = "CONFLICT";
     }else if( (flags & (C_EDITED | C_CHANGED)) && isChnged
@@ -419,7 +419,6 @@ static int determine_cwd_relative_option()
 **    --case-sensitive <BOOL>  Override case-sensitive setting.
 **    --dotfiles        Include unmanaged files beginning with a dot.
 **    --ignore <CSG>    Ignore unmanaged files matching CSG glob patterns.
-**    --no-dir-symlinks Disables support for directory symlinks.
 **
 ** Options specific to the changes command:
 **    --header          Identify the repository if report is non-empty.
@@ -475,6 +474,8 @@ void status_cmd(void){
   unsigned flags = 0;
   int vid, i;
 
+  fossil_pledge("stdio rpath wpath cpath id flock tty chown");
+
   /* Load affirmative flag options. */
   for( i=0; i<count(flagDefs); ++i ){
     if( (command==CHANGES || !(flagDefs[i].mask & C_CLASSIFY))
@@ -767,7 +768,7 @@ void ls_cmd(void){
         type = "ADDED      ";
       }else if( isDeleted ){
         type = "DELETED    ";
-      }else if( !file_wd_isfile_or_link(zFullName) ){
+      }else if( !file_isfile_or_link(zFullName) ){
         if( file_access(zFullName, F_OK)==0 ){
           type = "NOT_A_FILE ";
         }else{
@@ -828,7 +829,6 @@ void ls_cmd(void){
 **    --dotfiles       include files beginning with a dot (".")
 **    --header         Identify the repository if there are extras
 **    --ignore <CSG>   ignore files matching patterns from the argument
-**    --no-dir-symlinks Disables support for directory symlinks.
 **    --rel-paths      Display pathnames relative to the current working
 **                     directory.
 **
@@ -858,6 +858,8 @@ void extras_cmd(void){
     zIgnoreFlag = db_get("ignore-glob", 0);
   }
   pIgnore = glob_create(zIgnoreFlag);
+  /* Always consider symlinks. */
+  g.allowSymlinks = db_allow_symlinks_by_default();
   locate_unmanaged_files(g.argc-2, g.argv+2, scanFlags, pIgnore);
   glob_free(pIgnore);
 
@@ -903,10 +905,9 @@ void extras_cmd(void){
 ** the (versionable) clean-glob, ignore-glob, and keep-glob settings.
 **
 ** The --verily option ignores the keep-glob and ignore-glob settings and
-** turns on the options --force, --emptydirs, --dotfiles, --disable-undo,
-** and --no-dir-symlinks.  Use the --verily option when you really want
-** to clean up everything.  Extreme care should be exercised when using
-** the --verily option.
+** turns on --force, --emptydirs, --dotfiles, and --disable-undo.  Use the
+** --verily option when you really want to clean up everything.  Extreme
+** care should be exercised when using the --verily option.
 **
 ** Options:
 **    --allckouts      Check for empty directories within any checkouts
@@ -936,7 +937,7 @@ void extras_cmd(void){
 **    -x|--verily      WARNING: Removes everything that is not a managed
 **                     file or the repository itself.  This option
 **                     implies the --force, --emptydirs, --dotfiles, and
-**                     --disable-undo, and --no-dir-symlinks options.
+**                     --disable-undo options.
 **                     Furthermore, it completely disregards the keep-glob
 **                     and ignore-glob settings.  However, it does honor
 **                     the --ignore and --keep options.
@@ -952,7 +953,6 @@ void extras_cmd(void){
 **                     deleted.
 **    --no-prompt      This option disables prompting the user for input
 **                     and assumes an answer of 'No' for every question.
-**    --no-dir-symlinks Disables support for directory symlinks.
 **    --temp           Remove only Fossil-generated temporary files.
 **    -v|--verbose     Show all files as they are removed.
 **
@@ -1001,8 +1001,6 @@ void clean_cmd(void){
     disableUndo = 1;
     scanFlags |= SCAN_ALL;
     zCleanFlag = 0;
-    g.fNoDirSymlinks = 1; /* Forbid symlink directory traversal. */
-    g.allowSymlinks = 1;  /* Treat symlink files as content. */
   }
   if( zIgnoreFlag==0 && !verilyFlag ){
     zIgnoreFlag = db_get("ignore-glob", 0);
@@ -1019,6 +1017,8 @@ void clean_cmd(void){
   pKeep = glob_create(zKeepFlag);
   pClean = glob_create(zCleanFlag);
   nRoot = (int)strlen(g.zLocalRoot);
+  /* Always consider symlinks. */
+  g.allowSymlinks = db_allow_symlinks_by_default();
   if( !dirsOnlyFlag ){
     Stmt q;
     Blob repo;
@@ -1204,10 +1204,14 @@ void prompt_for_user_comment(Blob *pComment, Blob *pPrompt){
   }else{
     Blob fname;
     blob_zero(&fname);
-    assert( g.zLocalRoot!=0 );
-    file_relative_name(g.zLocalRoot, &fname, 1);
-    zFile = db_text(0, "SELECT '%qci-comment-'||hex(randomblob(6))||'.txt'",
-                    blob_str(&fname));
+    if( g.zLocalRoot!=0 ){
+      file_relative_name(g.zLocalRoot, &fname, 1);
+      zFile = db_text(0, "SELECT '%qci-comment-'||hex(randomblob(6))||'.txt'",
+                      blob_str(&fname));
+    }else{
+      file_tempname(&fname, "ci-comment");
+      zFile = mprintf("%s", blob_str(&fname));
+    }
     blob_reset(&fname);
   }
 #if defined(_WIN32)
@@ -1221,7 +1225,7 @@ void prompt_for_user_comment(Blob *pComment, Blob *pPrompt){
       fossil_fatal("editor aborted: \"%s\"", zCmd);
     }
 
-    blob_read_from_file(&reply, zFile);
+    blob_read_from_file(&reply, zFile, ExtFILE);
   }else{
     char zIn[300];
     blob_zero(&reply);
@@ -1562,7 +1566,7 @@ static void create_manifest(
     if( isSelected ){
       int mPerm;
 
-      mPerm = file_wd_perm(blob_str(&filename));
+      mPerm = file_perm(blob_str(&filename), RepoFILE);
       isExe = ( mPerm==PERM_EXE );
       isLink = ( mPerm==PERM_LNK );
     }
@@ -1908,11 +1912,7 @@ void test_commit_warning(void){
     binOk = db_column_int(&q, 3);
     encodingOk = db_column_int(&q, 4);
     blob_zero(&content);
-    if( file_wd_islink(zFullname) ){
-      blob_read_link(&content, zFullname);
-    }else{
-      blob_read_from_file(&content, zFullname);
-    }
+    blob_read_from_file(&content, zFullname, RepoFILE);
     blob_zero(&reason);
     fileRc = commit_warning(&content, crlfOk, binOk, encodingOk, 2,
                             zFullname, &reason);
@@ -2303,7 +2303,7 @@ void commit_cmd(void){
     blob_append(&comment, zComment, -1);
   }else if( zComFile ){
     blob_zero(&comment);
-    blob_read_from_file(&comment, zComFile);
+    blob_read_from_file(&comment, zComFile, ExtFILE);
     blob_to_utf8_no_bom(&comment, 1);
   }else if( dryRunFlag ){
     blob_zero(&comment);
@@ -2374,12 +2374,7 @@ void commit_cmd(void){
     encodingOk = db_column_int(&q, 5);
 
     blob_zero(&content);
-    if( file_wd_islink(zFullname) ){
-      /* Instead of file content, put link destination path */
-      blob_read_link(&content, zFullname);
-    }else{
-      blob_read_from_file(&content, zFullname);
-    }
+    blob_read_from_file(&content, zFullname, RepoFILE);
     /* Do not emit any warnings when they are disabled. */
     if( !noWarningFlag ){
       abortCommit |= commit_warning(&content, crlfOk, binOk,
@@ -2398,7 +2393,7 @@ void commit_cmd(void){
     nrid = content_put(&content);
     blob_reset(&content);
     if( rid>0 ){
-      content_deltify(rid, nrid, 0);
+      content_deltify(rid, &nrid, 1, 0);
     }
     db_multi_exec("UPDATE vfile SET mrid=%d, rid=%d WHERE id=%d", nrid,nrid,id);
     db_multi_exec("INSERT OR IGNORE INTO unsent VALUES(%d)", nrid);
@@ -2491,7 +2486,7 @@ void commit_cmd(void){
     zManifestFile = mprintf("%smanifest", g.zLocalRoot);
     blob_write_to_file(&manifest, zManifestFile);
     blob_reset(&manifest);
-    blob_read_from_file(&manifest, zManifestFile);
+    blob_read_from_file(&manifest, zManifestFile, ExtFILE);
     free(zManifestFile);
   }
 
@@ -2505,7 +2500,7 @@ void commit_cmd(void){
     fossil_fatal("%s", g.zErrMsg);
   }
   assert( blob_is_reset(&manifest) );
-  content_deltify(vid, nvid, 0);
+  content_deltify(vid, &nvid, 1, 0);
   zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", nvid);
 
   db_prepare(&q, "SELECT uuid,merge FROM vmerge JOIN blob ON merge=rid"
