@@ -37,12 +37,12 @@ static struct {
 **              It will always begin with a ':'.
 **   -rid: The unique object ID that identifies this commit within the
 **         repository database.
-**   -uuid: The SHA-1 of artifact corresponding to rid.
+**   -uuid: The SHA-1/SHA-3 of artifact corresponding to rid.
 */
 struct mark_t{
   char *name;
   int rid;
-  char uuid[41];
+  char uuid[65];
 };
 #endif
 
@@ -342,9 +342,9 @@ int parse_mark(char *line, struct mark_t *mark){
   }
 
   cur_tok = strtok(NULL, "\n");
-  if( !cur_tok || strlen(cur_tok)!=40 ){
+  if( !cur_tok || (strlen(cur_tok)!=40 && strlen(cur_tok)!=64) ){
     free(mark->name);
-    fossil_trace("Invalid SHA-1 in marks file: %s\n", cur_tok);
+    fossil_trace("Invalid SHA-1/SHA-3 in marks file: %s\n", cur_tok);
     return -1;
   }else{
     sqlite3_snprintf(sizeof(mark->uuid), mark->uuid, "%s", cur_tok);
@@ -353,7 +353,7 @@ int parse_mark(char *line, struct mark_t *mark){
   /* make sure that rid corresponds to UUID */
   if( fast_uuid_to_rid(mark->uuid)!=mark->rid ){
     free(mark->name);
-    fossil_trace("Non-existent SHA-1 in marks file: %s\n", mark->uuid);
+    fossil_trace("Non-existent SHA-1/SHA-3 in marks file: %s\n", mark->uuid);
     return -1;
   }
 
@@ -487,7 +487,6 @@ void export_marks(FILE* f, Bag *blobs, Bag *vers){
 */
 void export_cmd(void){
   Stmt q, q2, q3;
-  int i;
   Bag blobs, vers;
   unsigned int unused_mark = 1;
   const char *markfile_in;
@@ -511,6 +510,7 @@ void export_cmd(void){
   db_multi_exec("CREATE TEMPORARY TABLE oldblob(rid INTEGER PRIMARY KEY)");
   db_multi_exec("CREATE TEMPORARY TABLE oldcommit(rid INTEGER PRIMARY KEY)");
   db_multi_exec("CREATE TEMP TABLE xmark(tname TEXT UNIQUE, trid INT, tuuid TEXT)");
+  db_multi_exec("CREATE INDEX xmark_trid ON xmark(trid)");
   if( markfile_in!=0 ){
     Stmt qb,qc;
     FILE *f;
@@ -720,7 +720,7 @@ void export_cmd(void){
     printf("tagger");
     print_person(zUser);
     printf(" %s +0000\n", zSecSince1970);
-    printf("data %d\n", zValue==NULL?0:strlen(zValue)+1);
+    printf("data %d\n", zValue==NULL?0:(int)strlen(zValue)+1);
     if( zValue!=NULL ) printf("%s\n",zValue);
   }
   db_finalize(&q);
