@@ -20,7 +20,7 @@
 **
 ** For notes regarding the public JSON interface, please see:
 **
-** https://docs.google.com/document/d/1fXViveNhDbiXgCuE7QDXQOKeFzf2qNUkBEgiUvoqFN4/edit
+** https://docs.google.com/document/d/1fXViveNhDbiXgCuE7QDXQOKeFzf2qNUkBEgiUvoqFN4/view
 **
 **
 ** Notes for hackers...
@@ -32,8 +32,8 @@
 **
 **
 */
-#include "config.h"
 #include "VERSION.h"
+#include "config.h"
 #include "json.h"
 #include <assert.h>
 #include <time.h>
@@ -55,132 +55,11 @@ const FossilJsonKeys_ FossilJsonKeys = {
 };
 
 
-/* Timer code taken from sqlite3's shell.c, modified slightly.
-   FIXME: move the timer into the fossil core API so that we can
-   start the timer early on in the app init phase. Right now we're
-   just timing the json ops themselves.
-*/
-#if !defined(_WIN32) && !defined(WIN32) && !defined(__OS2__) && !defined(__RTP__) && !defined(_WRS_KERNEL)
-#include <sys/time.h>
-#include <sys/resource.h>
-
-/* Saved resource information for the beginning of an operation */
-static struct rusage sBegin;
-
-/*
-** Begin timing an operation
-*/
-static void beginTimer(void){
-  getrusage(RUSAGE_SELF, &sBegin);
-}
-
-/* Return the difference of two time_structs in milliseconds */
-static double timeDiff(struct timeval *pStart, struct timeval *pEnd){
-  return ((pEnd->tv_usec - pStart->tv_usec)*0.001 +
-          (double)((pEnd->tv_sec - pStart->tv_sec)*1000.0));
-}
-
-/*
-** Print the timing results.
-*/
-static double endTimer(void){
-  struct rusage sEnd;
-  getrusage(RUSAGE_SELF, &sEnd);
-#if 0
-  printf("CPU Time: user %f sys %f\n",
-         timeDiff(&sBegin.ru_utime, &sEnd.ru_utime),
-         timeDiff(&sBegin.ru_stime, &sEnd.ru_stime));
-#endif
-  return timeDiff(&sBegin.ru_utime, &sEnd.ru_utime)
-    + timeDiff(&sBegin.ru_stime, &sEnd.ru_stime);
-}
-
-#define BEGIN_TIMER beginTimer()
-#define END_TIMER endTimer()
-#define HAS_TIMER 1
-
-#elif (defined(_WIN32) || defined(WIN32))
-
-#include <windows.h>
-
-/* Saved resource information for the beginning of an operation */
-static HANDLE hProcess;
-static FILETIME ftKernelBegin;
-static FILETIME ftUserBegin;
-typedef BOOL (WINAPI *GETPROCTIMES)(HANDLE, LPFILETIME, LPFILETIME, LPFILETIME, LPFILETIME);
-static GETPROCTIMES getProcessTimesAddr = NULL;
-
-/*
-** Check to see if we have timer support.  Return 1 if necessary
-** support found (or found previously).
-*/
-static int hasTimer(void){
-  if( getProcessTimesAddr ){
-    return 1;
-  } else {
-    /* GetProcessTimes() isn't supported in WIN95 and some other Windows versions.
-    ** See if the version we are running on has it, and if it does, save off
-    ** a pointer to it and the current process handle.
-    */
-    hProcess = GetCurrentProcess();
-    if( hProcess ){
-      HINSTANCE hinstLib = LoadLibrary(TEXT("Kernel32.dll"));
-      if( NULL != hinstLib ){
-        getProcessTimesAddr = (GETPROCTIMES) GetProcAddress(hinstLib, "GetProcessTimes");
-        if( NULL != getProcessTimesAddr ){
-          return 1;
-        }
-        FreeLibrary(hinstLib);
-      }
-    }
-  }
-  return 0;
-}
-
-/*
-** Begin timing an operation
-*/
-static void beginTimer(void){
-  if( getProcessTimesAddr ){
-    FILETIME ftCreation, ftExit;
-    getProcessTimesAddr(hProcess, &ftCreation, &ftExit, &ftKernelBegin, &ftUserBegin);
-  }
-}
-
-/* Return the difference of two FILETIME structs in milliseconds */
-static double timeDiff(FILETIME *pStart, FILETIME *pEnd){
-  sqlite_int64 i64Start = *((sqlite_int64 *) pStart);
-  sqlite_int64 i64End = *((sqlite_int64 *) pEnd);
-  return (double) ((i64End - i64Start) / 10000.0);
-}
-
-/*
-** Print the timing results.
-*/
-static double endTimer(void){
-  if(getProcessTimesAddr){
-    FILETIME ftCreation, ftExit, ftKernelEnd, ftUserEnd;
-    getProcessTimesAddr(hProcess, &ftCreation, &ftExit, &ftKernelEnd, &ftUserEnd);
-    return timeDiff(&ftUserBegin, &ftUserEnd) +
-      timeDiff(&ftKernelBegin, &ftKernelEnd);
-  }
-  return 0.0;
-}
-
-#define BEGIN_TIMER beginTimer()
-#define END_TIMER endTimer()
-#define HAS_TIMER hasTimer()
-
-#else
-#define BEGIN_TIMER
-#define END_TIMER 0.0
-#define HAS_TIMER 0
-#endif
 
 /*
 ** Returns true (non-0) if fossil appears to be running in JSON mode.
 */
-char fossil_has_json(){
+int fossil_has_json(){
   return g.json.isJsonMode && (g.isHTTP || g.json.post.o);
 }
 
@@ -241,6 +120,7 @@ static char const * json_err_cstr( int errCode ){
     C(DB_NEEDS_REBUILD,"Fossil repository needs to be rebuilt");
     C(DB_NOT_FOUND,"Fossil repository db file could not be found.");
     C(DB_NOT_VALID, "Fossil repository db file is not valid.");
+    C(DB_NEEDS_CHECKOUT, "Command requires a local checkout.");
 #undef C
     default:
       return "Unknown Error";
@@ -324,7 +204,7 @@ char const * json_rc_cstr( int code ){
 }
 
 /*
-** Adds v to the API-internal cleanup mechanism. key is ingored
+** Adds v to the API-internal cleanup mechanism. key is ignored
 ** (legacy) but might be re-introduced and "should" be a unique
 ** (app-wide) value.  Failure to insert an item may be caused by any
 ** of the following:
@@ -344,6 +224,7 @@ char const * json_rc_cstr( int code ){
 */
 void json_gc_add( char const * key, cson_value * v ){
   int const rc = cson_array_append( g.json.gc.a, v );
+
   assert( NULL != g.json.gc.a );
   if( 0 != rc ){
     cson_value_free( v );
@@ -389,7 +270,7 @@ cson_value * json_new_string_f( char const * fmt, ... ){
   return v;
 }
 
-cson_value * json_new_int( int v ){
+cson_value * json_new_int( i64 v ){
   return cson_value_new_integer((cson_int_t)v);
 }
 
@@ -506,7 +387,7 @@ int json_getenv_int(char const * pKey, int dflt ){
 ** whether or not this function found a match (it will return -1 in
 ** that case).
 */
-char json_getenv_bool(char const * pKey, char dflt ){
+int json_getenv_bool(char const * pKey, int dflt ){
   cson_value const * v = json_getenv(pKey);
   const cson_type_id type = v ? cson_value_type_id(v) : CSON_TYPE_UNDEF;
   switch(type){
@@ -551,7 +432,7 @@ char const * json_getenv_cstr( char const * zKey ){
 ** GET/POST/CLI argument.
 **
 ** zKey must be the GET/POST parameter key. zCLILong must be the "long
-s** form" CLI flag (NULL means to use zKey). zCLIShort may be NULL or
+** form" CLI flag (NULL means to use zKey). zCLIShort may be NULL or
 ** the "short form" CLI flag (if NULL, no short form is used).
 **
 ** If argPos is >=0 and no other match is found,
@@ -597,11 +478,11 @@ char const * json_find_option_cstr(char const * zKey,
 ** The boolean equivalent of json_find_option_cstr().
 ** If the option is not found, dftl is returned.
 */
-char json_find_option_bool(char const * zKey,
-                           char const * zCLILong,
-                           char const * zCLIShort,
-                           char dflt ){
-  char rc = -1;
+int json_find_option_bool(char const * zKey,
+                          char const * zCLILong,
+                          char const * zCLIShort,
+                          int dflt ){
+  int rc = -1;
   if(!g.isHTTP){
     if(NULL != find_option(zCLILong ? zCLILong : zKey,
                            zCLIShort, 0)){
@@ -753,7 +634,8 @@ void json_send_response( cson_value const * pResponse ){
 ** The result of this call are cached for future calls.
 */
 cson_value * json_auth_token(){
-  if( !g.json.authToken ){
+    assert(g.json.gc.a && "json_main_bootstrap() was not called!");
+    if( !g.json.authToken ){
     /* Try to get an authorization token from GET parameter, POSTed
        JSON, or fossil cookie (in that order). */
     g.json.authToken = json_getenv(FossilJsonKeys.authToken);
@@ -815,7 +697,10 @@ cson_value * json_req_payload_get(char const *pKey){
 */
 void json_main_bootstrap(){
   cson_value * v;
-  assert( (NULL == g.json.gc.v) && "cgi_json_bootstrap() was called twice!" );
+  assert( (NULL == g.json.gc.v) &&
+          "json_main_bootstrap() was called twice!" );
+
+  g.json.timerId = fossil_timer_start();
 
   /* g.json.gc is our "garbage collector" - where we put JSON values
      which need a long lifetime but don't have a logical parent to put
@@ -823,10 +708,12 @@ void json_main_bootstrap(){
   */
   v = cson_value_new_array();
   g.json.gc.v = v;
+  assert(0 != g.json.gc.v);
   g.json.gc.a = cson_value_get_array(v);
+  assert(0 != g.json.gc.a);
   cson_value_add_reference(v)
     /* Needed to allow us to include this value in other JSON
-       containers without transfering ownership to those containers.
+       containers without transferring ownership to those containers.
        All other persistent g.json.XXX.v values get appended to
        g.json.gc.a, and therefore already have a live reference
        for this purpose.
@@ -874,6 +761,7 @@ void json_warn( int code, char const * fmt, ... ){
   assert( (code>FSL_JSON_W_START)
           && (code<FSL_JSON_W_END)
           && "Invalid warning code.");
+  assert(g.json.gc.a && "json_main_bootstrap() was not called!");
   if(!g.json.warnings){
     g.json.warnings = cson_new_array();
     assert((NULL != g.json.warnings) && "Alloc error.");
@@ -920,7 +808,7 @@ void json_warn( int code, char const * fmt, ... ){
 */
 int json_string_split( char const * zStr,
                        char separator,
-                       char doDeHttp,
+                       int doDeHttp,
                        cson_array * target ){
   char const * p = zStr /* current byte */;
   char const * head  /* current start-of-token */;
@@ -995,7 +883,7 @@ int json_string_split( char const * zStr,
 */
 cson_value * json_string_split2( char const * zStr,
                                  char separator,
-                                 char doDeHttp ){
+                                 int doDeHttp ){
   cson_array * a = cson_new_array();
   int rc = json_string_split( zStr, separator, doDeHttp, a );
   if( 0>=rc ){
@@ -1023,6 +911,7 @@ cson_value * json_string_split2( char const * zStr,
 static void json_mode_bootstrap(){
   static char once = 0  /* guard against multiple runs */;
   char const * zPath = P("PATH_INFO");
+  assert(g.json.gc.a && "json_main_bootstrap() was not called!");
   assert( (0==once) && "json_mode_bootstrap() called too many times!");
   if( once ){
     return;
@@ -1106,7 +995,7 @@ static void json_mode_bootstrap(){
     }
     inFile = (0==strcmp("-",jfile))
       ? stdin
-      : fopen(jfile,"rb");
+      : fossil_fopen(jfile,"rb");
     if(!inFile){
       g.json.resultCode = FSL_JSON_E_FILE_OPEN_FAILED;
       fossil_fatal("Could not open JSON file [%s].",jfile)
@@ -1201,7 +1090,7 @@ static void json_mode_bootstrap(){
 ** find_option() to get those.
 **
 */
-char const * json_command_arg(unsigned char ndx){
+char const * json_command_arg(unsigned short ndx){
   cson_array * ar = g.json.cmd.a;
   assert((NULL!=ar) && "Internal error. Was json_mode_bootstrap() called?");
   assert((g.argc>1) && "Internal error - we never should have gotten this far.");
@@ -1372,8 +1261,7 @@ cson_value * json_g_to_json(){
   VAL(capabilities, json_cap_value());
   INT(g, argc);
   INT(g, isConst);
-  INT(g, useAttach);
-  INT(g, configOpen);
+  CSTR(g, zConfigDbName);
   INT(g, repositoryOpen);
   INT(g, localOpen);
   INT(g, minPrefix);
@@ -1394,12 +1282,11 @@ cson_value * json_g_to_json(){
   INT(g, markPrivate);
   INT(g, clockSkewSeen);
   INT(g, isHTTP);
-  INT(g, urlIsFile);
-  INT(g, urlIsHttps);
-  INT(g, urlIsSsh);
-  INT(g, urlPort);
-  INT(g, urlDfltPort);
-  INT(g, dontKeepUrl);
+  INT(g.url, isFile);
+  INT(g.url, isHttps);
+  INT(g.url, isSsh);
+  INT(g.url, port);
+  INT(g.url, dfltPort);
   INT(g, useLocalauth);
   INT(g, noPswd);
   INT(g, userUid);
@@ -1410,8 +1297,7 @@ cson_value * json_g_to_json(){
   INT(g, nAux);
   INT(g, allowSymlinks);
 
-  CSTR(g, zMainDbType);
-  CSTR(g, zHome);
+  CSTR(g, zOpenRevision);
   CSTR(g, zLocalRoot);
   CSTR(g, zPath);
   CSTR(g, zExtra);
@@ -1419,15 +1305,15 @@ cson_value * json_g_to_json(){
   CSTR(g, zTop);
   CSTR(g, zContentType);
   CSTR(g, zErrMsg);
-  CSTR(g, urlName);
-  CSTR(g, urlHostname);
-  CSTR(g, urlProtocol);
-  CSTR(g, urlPath);
-  CSTR(g, urlUser);
-  CSTR(g, urlPasswd);
-  CSTR(g, urlCanonical);
-  CSTR(g, urlProxyAuth);
-  CSTR(g, urlFossil);
+  CSTR(g.url, name);
+  CSTR(g.url, hostname);
+  CSTR(g.url, protocol);
+  CSTR(g.url, path);
+  CSTR(g.url, user);
+  CSTR(g.url, passwd);
+  CSTR(g.url, canonical);
+  CSTR(g.url, proxyAuth);
+  CSTR(g.url, fossil);
   CSTR(g, zLogin);
   CSTR(g, zSSLIdentity);
   CSTR(g, zIpAddr);
@@ -1468,7 +1354,7 @@ cson_value * json_g_to_json(){
 ** it defaults to g.json.resultCode. If resultCode is (or defaults to)
 ** non-zero and payload is not NULL then this function calls
 ** cson_value_free(payload) and does not insert the payload into the
-** response. In either case, onwership of payload is transfered to (or
+** response. In either case, ownership of payload is transfered to (or
 ** shared with, if the caller holds a reference) this function.
 **
 ** pMsg is an optional message string property (resultText) of the
@@ -1539,25 +1425,29 @@ static cson_value * json_create_response( int resultCode,
       tmp = g.json.param.v;
       SET("$params");
     }
-    if(0){/*Only for debuggering, add some info to the response.*/
+    if(0){/*Only for debugging, add some info to the response.*/
       tmp = cson_value_new_integer( g.json.cmd.offset );
       cson_object_set( o, "cmd.offset", tmp );
       cson_object_set( o, "isCGI", cson_value_new_bool( g.isHTTP ) );
     }
   }
 
-  if(HAS_TIMER){
+  if(fossil_timer_is_active(g.json.timerId)){
     /* This is, philosophically speaking, not quite the right place
        for ending the timer, but this is the one function which all of
        the JSON exit paths use (and they call it after processing,
        just before they end).
     */
-    double span;
-    span = END_TIMER;
-    /* i'm actually seeing sub-ms runtimes in some tests, but a time of
-       0 is "just wrong", so we'll bump that up to 1ms.
+    sqlite3_uint64 span = fossil_timer_stop(g.json.timerId);
+    /* I'm actually seeing sub-uSec runtimes in some tests, but a time of
+       0 is "just kinda wrong".
     */
-    cson_object_set(o,"procTimeMs", cson_value_new_integer((cson_int_t)((span>1.0)?span:1)));
+    cson_object_set(o,"procTimeUs", cson_value_new_integer((cson_int_t)span));
+    span /= 1000/*for milliseconds */;
+    cson_object_set(o,"procTimeMs", cson_value_new_integer((cson_int_t)span));
+    assert(!fossil_timer_is_active(g.json.timerId));
+    g.json.timerId = -1;
+
   }
   if(g.json.warnings){
     tmp = cson_array_value(g.json.warnings);
@@ -1612,7 +1502,7 @@ static cson_value * json_create_response( int resultCode,
 ** is used as-is. If it is NULL then g.zErrMsg is checked, and if that
 ** is NULL then json_err_cstr(code) is used.
 */
-void json_err( int code, char const * msg, char alsoOutput ){
+void json_err( int code, char const * msg, int alsoOutput ){
   int rc = code ? code : (g.json.resultCode
                           ? g.json.resultCode
                           : FSL_JSON_E_UNKNOWN);
@@ -1720,7 +1610,7 @@ cson_value * json_stmt_to_array_of_obj(Stmt *pStmt,
   }
   cson_value_free(colNamesV);
   if(warnMsg){
-    json_warn( FSL_JSON_W_ROW_TO_JSON_FAILED, warnMsg );
+    json_warn( FSL_JSON_W_ROW_TO_JSON_FAILED, "%s", warnMsg );
   }
   return cson_array_value(a);
 }
@@ -1777,11 +1667,11 @@ cson_value * json_stmt_to_array_of_values(Stmt *pStmt,
 ** to simplify the trivial use-cases (which don't need a Blob).
 */
 cson_value * json_sql_to_array_of_obj(Blob * pSql, cson_array * pTgt,
-                                      char resetBlob){
+                                      int resetBlob){
   Stmt q = empty_Stmt;
   cson_value * pay = NULL;
   assert( blob_size(pSql) > 0 );
-  db_prepare(&q, "%s", blob_str(pSql));
+  db_prepare(&q, "%s", blob_str(pSql) /*safe-for-%s*/);
   if(resetBlob){
     blob_reset(pSql);
   }
@@ -1802,7 +1692,7 @@ cson_value * json_sql_to_array_of_obj(Blob * pSql, cson_array * pTgt,
 ** If there are no tags then this function returns NULL, not an empty
 ** Array.
 */
-cson_value * json_tags_for_checkin_rid(int rid, char propagatingOnly){
+cson_value * json_tags_for_checkin_rid(int rid, int propagatingOnly){
   cson_value * v = NULL;
   char * tags = info_tags_of_checkin(rid, propagatingOnly);
   if(tags){
@@ -1991,10 +1881,12 @@ cson_value * json_page_cap(){
   ADD(NewWiki,"createWiki");
   ADD(ApndWiki,"appendWiki");
   ADD(WrWiki,"editWiki");
+  ADD(ModWiki,"moderateWiki");
   ADD(RdTkt,"readTicket");
   ADD(NewTkt,"createTicket");
   ADD(ApndTkt,"appendTicket");
   ADD(WrTkt,"editTicket");
+  ADD(ModTkt,"moderateTicket");
   ADD(Attach,"attachFile");
   ADD(TktFmt,"createTicketReport");
   ADD(RdAddr,"readPrivate");
@@ -2012,7 +1904,6 @@ cson_value * json_page_stat(){
   i64 t, fsize;
   int n, m;
   int full;
-  const char *zDb;
   enum { BufLen = 1000 };
   char zBuf[BufLen];
   cson_value * jv = NULL;
@@ -2025,7 +1916,8 @@ cson_value * json_page_stat(){
                  "Requires 'o' permissions.");
     return NULL;
   }
-  full = json_find_option_bool("full",NULL,"f",0);
+  full = json_find_option_bool("full",NULL,"f",
+              json_find_option_bool("verbose",NULL,"v",0));
 #define SETBUF(O,K) cson_object_set(O, K, cson_value_new_string(zBuf, strlen(zBuf)));
 
   jv = cson_value_new_object();
@@ -2038,8 +1930,9 @@ cson_value * json_page_stat(){
   cson_object_set(jo, "projectDescription", json_new_string(zTmp));
   free(zTmp);
   zTmp = NULL;
-  fsize = file_size(g.zRepositoryName);
-  cson_object_set(jo, "repositorySize", cson_value_new_integer((cson_int_t)fsize));
+  fsize = file_size(g.zRepositoryName, ExtFILE);
+  cson_object_set(jo, "repositorySize", 
+                  cson_value_new_integer((cson_int_t)fsize));
 
   if(full){
     n = db_int(0, "SELECT count(*) FROM blob");
@@ -2084,26 +1977,23 @@ cson_value * json_page_stat(){
   n = db_int(0, "SELECT julianday('now') - (SELECT min(mtime) FROM event)"
                 " + 0.99");
   cson_object_set(jo, "ageDays", cson_value_new_integer((cson_int_t)n));
-  cson_object_set(jo, "ageYears", cson_value_new_double(n/365.24));
+  cson_object_set(jo, "ageYears", cson_value_new_double(n/365.2425));
   sqlite3_snprintf(BufLen, zBuf, db_get("project-code",""));
   SETBUF(jo, "projectCode");
-  sqlite3_snprintf(BufLen, zBuf, db_get("server-code",""));
-  SETBUF(jo, "serverCode");
   cson_object_set(jo, "compiler", cson_value_new_string(COMPILER_NAME, strlen(COMPILER_NAME)));
 
   jv2 = cson_value_new_object();
   jo2 = cson_value_get_object(jv2);
   cson_object_set(jo, "sqlite", jv2);
   sqlite3_snprintf(BufLen, zBuf, "%.19s [%.10s] (%s)",
-                   SQLITE_SOURCE_ID, &SQLITE_SOURCE_ID[20], SQLITE_VERSION);
+                   sqlite3_sourceid(), &sqlite3_sourceid()[20], sqlite3_libversion());
   SETBUF(jo2, "version");
-  zDb = db_name("repository");
-  cson_object_set(jo2, "pageCount", cson_value_new_integer((cson_int_t)db_int(0, "PRAGMA %s.page_count", zDb)));
-  cson_object_set(jo2, "pageSize", cson_value_new_integer((cson_int_t)db_int(0, "PRAGMA %s.page_size", zDb)));
-  cson_object_set(jo2, "freeList", cson_value_new_integer((cson_int_t)db_int(0, "PRAGMA %s.freelist_count", zDb)));
-  sqlite3_snprintf(BufLen, zBuf, "%s", db_text(0, "PRAGMA %s.encoding", zDb));
+  cson_object_set(jo2, "pageCount", cson_value_new_integer((cson_int_t)db_int(0, "PRAGMA repository.page_count")));
+  cson_object_set(jo2, "pageSize", cson_value_new_integer((cson_int_t)db_int(0, "PRAGMA repository.page_size")));
+  cson_object_set(jo2, "freeList", cson_value_new_integer((cson_int_t)db_int(0, "PRAGMA repository.freelist_count")));
+  sqlite3_snprintf(BufLen, zBuf, "%s", db_text(0, "PRAGMA repository.encoding"));
   SETBUF(jo2, "encoding");
-  sqlite3_snprintf(BufLen, zBuf, "%s", db_text(0, "PRAGMA %s.journal_mode", zDb));
+  sqlite3_snprintf(BufLen, zBuf, "%s", db_text(0, "PRAGMA repository.journal_mode"));
   cson_object_set(jo2, "journalMode", *zBuf ? cson_value_new_string(zBuf, strlen(zBuf)) : cson_value_null());
   return jv;
 #undef SETBUF
@@ -2119,15 +2009,19 @@ cson_value * json_page_stat(){
 ** are undefined.
 **
 ** The list is appended to pOut. The number of items (not bytes)
-** appended are returned.
+** appended are returned. If filterByMode is non-0 then the result
+** list will contain only commands which are able to run in the
+** current run mode (CLI vs. HTTP).
 */
 static int json_pagedefs_to_string(JsonPageDef const * zPages,
-                                   Blob * pOut){
+                                   Blob * pOut, int filterByMode){
   int i = 0;
   for( ; zPages->name; ++zPages, ++i ){
-    if(g.isHTTP && zPages->runMode < 0) continue;
-    else if(zPages->runMode > 0) continue;
-    blob_appendf(pOut, zPages->name, -1);
+    if(filterByMode){
+      if(g.isHTTP && zPages->runMode < 0) continue;
+      else if(zPages->runMode > 0) continue;
+    }
+    blob_append(pOut, zPages->name, -1);
     if((zPages+1)->name){
       blob_append(pOut, ", ",2);
     }
@@ -2154,7 +2048,7 @@ void json_dispatch_missing_args_err( JsonPageDef const * pCommands,
     zErrPrefix = "Try one of: ";
   }
   blob_append( &cmdNames, zErrPrefix, strlen(zErrPrefix) );
-  json_pagedefs_to_string(pCommands, &cmdNames);
+  json_pagedefs_to_string(pCommands, &cmdNames, 1);
   json_set_err(FSL_JSON_E_MISSING_ARGS, "%s",
                blob_str(&cmdNames));
   blob_reset(&cmdNames);
@@ -2184,7 +2078,7 @@ cson_value * json_page_dispatch_helper(JsonPageDef const * pages){
 
 
 /*
-** Impl of /json/rebuild. Requires admin previleges.
+** Impl of /json/rebuild. Requires admin privileges.
 */
 static cson_value * json_page_rebuild(){
   if( !g.perm.Admin ){
@@ -2247,6 +2141,8 @@ cson_value * json_page_user();
 cson_value * json_page_config();
 /* Impl in json_finfo.c. */
 cson_value * json_page_finfo();
+/* Impl in json_status.c. */
+cson_value * json_page_status();
 
 /*
 ** Mapping of names to JSON pages/commands.  Each name is a subpath of
@@ -2271,6 +2167,7 @@ static const JsonPageDef JsonPageDefs[] = {
 {"report", json_page_report, 0},
 {"resultCodes", json_page_resultCodes,0},
 {"stat",json_page_stat,0},
+{"status", json_page_status, 0},
 {"tag", json_page_tag,0},
 /*{"ticket", json_page_nyi,0},*/
 {"timeline", json_page_timeline,0},
@@ -2328,7 +2225,7 @@ static int json_dispatch_root_command( char const * zCommand ){
 */
 void json_page_top(void){
   char const * zCommand;
-  BEGIN_TIMER;
+  assert(g.json.gc.a && "json_main_bootstrap() was not called!");
   json_mode_bootstrap();
   zCommand = json_command_arg(1);
   if(!zCommand || !*zCommand){
@@ -2355,9 +2252,14 @@ void json_page_top(void){
 ** in the argument dispatching code, any -FLAGS must come after the final
 ** sub- (or subsub-) command.
 **
+** The -json-input FILE option can be used to read JSON data and process
+** it like the HTTP interface would. For example:
+**
+**   %fossil json -json-input my.json
+**
 ** The commands include:
 **
-**   anonymousPassord
+**   anonymousPassword
 **   artifact
 **   branch
 **   cap
@@ -2386,7 +2288,6 @@ void json_page_top(void){
 void json_cmd_top(void){
   char const * cmd = NULL;
   int rc = 0;
-  BEGIN_TIMER;
   memset( &g.perm, 0xff, sizeof(g.perm) )
     /* In CLI mode fossil does not use permissions
        and they all default to false. We enable them
