@@ -68,7 +68,7 @@ cson_value * json_page_timeline(){
 */
 static void json_timeline_temp_table(void){
   /* Field order MUST match that from json_timeline_query()!!! */
-  static const char zSql[] = 
+  static const char zSql[] =
     @ CREATE TEMP TABLE IF NOT EXISTS json_timeline(
     @   sortId INTEGER PRIMARY KEY,
     @   rid INTEGER,
@@ -85,22 +85,23 @@ static void json_timeline_temp_table(void){
     @   brief TEXT
     @ )
   ;
-  db_multi_exec(zSql);
+  db_multi_exec("%s", zSql /*safe-for-%s*/);
 }
 
 /*
 ** Return a pointer to a constant string that forms the basis
-** for a timeline query for the JSON interface.
+** for a timeline query for the JSON interface. It MUST NOT
+** be used in a formatted string argument.
 */
-const char const * json_timeline_query(void){
+char const * json_timeline_query(void){
   /* Field order MUST match that from json_timeline_temp_table()!!! */
   static const char zBaseSql[] =
     @ SELECT
     @   NULL,
     @   blob.rid,
     @   uuid,
-    @   CAST(strftime('%%s',event.mtime) AS INTEGER),
-    @   datetime(event.mtime,'utc'),
+    @   CAST(strftime('%s',event.mtime) AS INTEGER),
+    @   datetime(event.mtime),
     @   coalesce(ecomment, comment),
     @   coalesce(euser, user),
     @   blob.rid IN leaf,
@@ -111,7 +112,7 @@ const char const * json_timeline_query(void){
     @       AND tagxref.rid=blob.rid AND tagxref.tagtype>0) as tags,
     @   tagid as tagId,
     @   brief as brief
-    @  FROM event JOIN blob 
+    @  FROM event JOIN blob
     @ WHERE blob.rid=event.objid
   ;
   return zBaseSql;
@@ -126,7 +127,7 @@ const char const * json_timeline_query(void){
 ** mode's "r" option. They are very similar, but subtly different -
 ** tag mode shows only entries with a given tag but branch mode can
 ** also reveal some with "related" tags (meaning they were merged into
-** the requested branch).
+** the requested branch, or back).
 **
 ** pSql is the target blob to append the query [subset]
 ** to.
@@ -146,6 +147,8 @@ static char json_timeline_add_tag_branch_clause(Blob *pSql,
                                                 cson_object * pPayload){
   char const * zTag = NULL;
   char const * zBranch = NULL;
+  char const * zMiOnly = NULL;
+  char const * zUnhide = NULL;
   int tagid = 0;
   if(! g.perm.Read ){
     return 0;
@@ -157,7 +160,9 @@ static char json_timeline_add_tag_branch_clause(Blob *pSql,
       return 0;
     }
     zTag = zBranch;
+    zMiOnly = json_find_option_cstr("mionly",NULL,NULL);
   }
+  zUnhide = json_find_option_cstr("unhide",NULL,NULL);
   tagid = db_int(0, "SELECT tagid FROM tag WHERE tagname='sym-%q'",
                  zTag);
   if(tagid<=0){
@@ -171,18 +176,36 @@ static char json_timeline_add_tag_branch_clause(Blob *pSql,
                " EXISTS(SELECT 1 FROM tagxref"
                "        WHERE tagid=%d AND tagtype>0 AND rid=blob.rid)",
                tagid);
+  if(!zUnhide){
+    blob_appendf(pSql,
+               " AND NOT EXISTS(SELECT 1 FROM plink JOIN tagxref ON rid=blob.rid"
+               "    WHERE tagid=%d AND tagtype>0 AND rid=blob.rid)",
+               TAG_HIDDEN);
+  }
   if(zBranch){
     /* from "r" flag code in page_timeline().*/
     blob_appendf(pSql,
                  " OR EXISTS(SELECT 1 FROM plink JOIN tagxref ON rid=cid"
                  "    WHERE tagid=%d AND tagtype>0 AND pid=blob.rid)",
                  tagid);
-#if 0 /* from the undocumented "mionly" flag in page_timeline() */
-    blob_appendf(pSql,
+    if( !zUnhide ){
+      blob_appendf(pSql,
+                 " AND NOT EXISTS(SELECT 1 FROM plink JOIN tagxref ON rid=cid"
+                 "    WHERE tagid=%d AND tagtype>0 AND pid=blob.rid)",
+                 TAG_HIDDEN);
+    }
+    if( zMiOnly==0 ){
+      blob_appendf(pSql,
                  " OR EXISTS(SELECT 1 FROM plink JOIN tagxref ON rid=pid"
                  "    WHERE tagid=%d AND tagtype>0 AND cid=blob.rid)",
                  tagid);
-#endif
+      if( !zUnhide ){
+        blob_appendf(pSql,
+                 " AND NOT EXISTS(SELECT 1 FROM plink JOIN tagxref ON rid=pid"
+                 "    WHERE tagid=%d AND tagtype>0 AND cid=blob.rid)",
+                 TAG_HIDDEN);
+      }
+    }
   }
   blob_append(pSql," ) ",3);
   return 1;
@@ -207,14 +230,14 @@ static char json_timeline_add_time_clause(Blob *pSql){
   if(zAfter&&*zAfter){
     while( fossil_isspace(*zAfter) ) ++zAfter;
     blob_appendf(pSql,
-                 " AND event.mtime>=(SELECT julianday(%Q,'utc')) "
+                 " AND event.mtime>=(SELECT julianday(%Q,fromLocal())) "
                  " ORDER BY event.mtime ASC ",
                  zAfter);
     rc = 1;
   }else if(zBefore && *zBefore){
     while( fossil_isspace(*zBefore) ) ++zBefore;
     blob_appendf(pSql,
-                 " AND event.mtime<=(SELECT julianday(%Q,'utc')) "
+                 " AND event.mtime<=(SELECT julianday(%Q,fromLocal())) "
                  " ORDER BY event.mtime DESC ",
                  zBefore);
     rc = -1;
@@ -296,7 +319,7 @@ cson_value * json_get_changed_files(int rid, int flags){
   cson_value * rowsV = NULL;
   cson_array * rows = NULL;
   Stmt q = empty_Stmt;
-  db_prepare(&q, 
+  db_prepare(&q,
            "SELECT (pid==0) AS isnew,"
            "       (fid==0) AS isdel,"
            "       (SELECT name FROM filename WHERE fnid=mlink.fnid) AS name,"
@@ -305,7 +328,7 @@ cson_value * json_get_changed_files(int rid, int flags){
            "       blob.size as size"
            "  FROM mlink, blob"
            " WHERE mid=%d AND pid!=fid"
-           " AND blob.rid=fid "
+           " AND blob.rid=fid AND NOT mlink.isaux"
            " ORDER BY name /*sort*/",
              rid
              );
@@ -364,7 +387,7 @@ static cson_value * json_timeline_branch(){
               " WHERE blob.rid=event.objid",
               -1);
 
-  blob_appendf(&sql,
+  blob_append_sql(&sql,
                " AND event.type='ci'"
                " AND blob.rid IN (SELECT rid FROM tagxref"
                "  WHERE tagtype>0 AND tagid=%d AND srcid!=0)"
@@ -372,9 +395,9 @@ static cson_value * json_timeline_branch(){
                TAG_BRANCH);
   limit = json_timeline_limit(20);
   if(limit>0){
-    blob_appendf(&sql," LIMIT %d ",limit);
+    blob_append_sql(&sql," LIMIT %d ",limit);
   }
-  db_prepare(&q,"%s", blob_str(&sql));
+  db_prepare(&q,"%s", blob_sql_text(&sql));
   blob_reset(&sql);
   pay = json_stmt_to_array_of_obj(&q, NULL);
   db_finalize(&q);
@@ -428,7 +451,7 @@ static cson_value * json_timeline_ci(){
   cson_value * listV = NULL;
   cson_array * list = NULL;
   int check = 0;
-  char showFiles = -1/*magic number*/;
+  char verboseFlag;
   Stmt q = empty_Stmt;
   char warnRowToJsonFailed = 0;
   Blob sql = empty_blob;
@@ -436,10 +459,13 @@ static cson_value * json_timeline_ci(){
     /* Reminder to self: HTML impl requires 'o' (Read)
        rights.
     */
-    json_set_err( FSL_JSON_E_DENIED, "Checkin timeline requires 'h' access." );
+    json_set_err( FSL_JSON_E_DENIED, "Check-in timeline requires 'h' access." );
     return NULL;
   }
-  showFiles = json_find_option_bool("files",NULL,"f",0);
+  verboseFlag = json_find_option_bool("verbose",NULL,"v",0);
+  if( !verboseFlag ){
+    verboseFlag = json_find_option_bool("files",NULL,"f",0);
+  }
   payV = cson_value_new_object();
   pay = cson_value_get_object(payV);
   check = json_timeline_setup_sql( "ci", &sql, pay );
@@ -459,7 +485,7 @@ static cson_value * json_timeline_ci(){
   tmp = cson_value_new_string(blob_buffer(&sql),strlen(blob_buffer(&sql)));
   SET("timelineSql");
 #endif
-  db_multi_exec(blob_buffer(&sql));
+  db_multi_exec("%s", blob_buffer(&sql)/*safe-for-%s*/);
   blob_reset(&sql);
   db_prepare(&q, "SELECT "
              " rid AS rid"
@@ -472,7 +498,7 @@ static cson_value * json_timeline_ci(){
   while( (SQLITE_ROW == db_step(&q) )){
     /* convert each row into a JSON object...*/
     int const rid = db_column_int(&q,0);
-    cson_value * rowV = json_artifact_for_ci(rid, showFiles);
+    cson_value * rowV = json_artifact_for_ci(rid, verboseFlag);
     cson_object * row = cson_value_get_object(rowV);
     if(!row){
       if( !warnRowToJsonFailed ){
@@ -521,10 +547,9 @@ cson_value * json_timeline_wiki(){
 
 #if 0
   /* only for testing! */
-  tmp = cson_value_new_string(blob_buffer(&sql),strlen(blob_buffer(&sql)));
-  SET("timelineSql");
+  cson_object_set(pay, "timelineSql", cson_value_new_string(blob_buffer(&sql),strlen(blob_buffer(&sql))));
 #endif
-  db_multi_exec(blob_buffer(&sql));
+  db_multi_exec("%s", blob_buffer(&sql) /*safe-for-%s*/);
   blob_reset(&sql);
   db_prepare(&q, "SELECT"
              " uuid AS uuid,"
@@ -542,8 +567,7 @@ cson_value * json_timeline_wiki(){
              " tagId AS tagId,"
 #endif
              " FROM json_timeline"
-             " ORDER BY rowid",
-             -1);
+             " ORDER BY rowid");
   list = cson_new_array();
   json_stmt_to_array_of_obj(&q, list);
   cson_object_set(pay, "timeline", cson_array_value(list));
@@ -584,7 +608,7 @@ static cson_value * json_timeline_ticket(){
     goto error;
   }
 
-  db_multi_exec(blob_buffer(&sql));
+  db_multi_exec("%s", blob_buffer(&sql) /*safe-for-%s*/);
 #define SET(K) if(0!=(check=cson_object_set(pay,K,tmp))){ \
     json_set_err((cson_rc.AllocError==check)        \
                  ? FSL_JSON_E_ALLOC : FSL_JSON_E_UNKNOWN,      \
@@ -615,8 +639,7 @@ static cson_value * json_timeline_ticket(){
              " comment AS comment,"
              " brief AS briefComment"
              " FROM json_timeline"
-             " ORDER BY rowid",
-             -1);
+             " ORDER BY rowid");
   listV = cson_value_new_array();
   list = cson_value_get_array(listV);
   tmp = listV;
@@ -629,9 +652,9 @@ static cson_value * json_timeline_ticket(){
     cson_value * rowV;
     cson_object * row;
     /*printf("rid=%d\n",rid);*/
-    pMan = manifest_get(rid, CFTYPE_TICKET);
+    pMan = manifest_get(rid, CFTYPE_TICKET, 0);
     if(!pMan){
-      /* this might be an attachment? i'm seeing this with
+      /* this might be an attachment? I'm seeing this with
          rid 15380, uuid [1292fef05f2472108].
 
          /json/artifact/1292fef05f2472108 returns not-found,
