@@ -143,7 +143,6 @@ struct Global {
   char *zLocalRoot;       /* The directory holding the  local database */
   int minPrefix;          /* Number of digits needed for a distinct UUID */
   int eHashPolicy;        /* Current hash policy.  One of HPOLICY_* */
-  int fNoDirSymlinks;     /* True if --no-dir-symlinks flag is present */
   int fSqlTrace;          /* True if --sqltrace flag is present */
   int fSqlStats;          /* True if --sqltrace or --sqlstats are present */
   int fSqlPrint;          /* True if -sqlprint flag is present */
@@ -234,10 +233,10 @@ struct Global {
   const char *azAuxVal[MX_AUX];  /* Value of each aux() or option() value */
   const char **azAuxOpt[MX_AUX]; /* Options of each option() value */
   int anAuxCols[MX_AUX];         /* Number of columns for option() values */
-
   int allowSymlinks;             /* Cached "allow-symlinks" option */
-
   int mainTimerId;               /* Set to fossil_timer_start() */
+  int nPendingRequest;           /* # of HTTP requests in "fossil server" */
+  int nRequest;                  /* Total # of HTTP request */
 #ifdef FOSSIL_ENABLE_JSON
   struct FossilJsonBits {
     int isJsonMode;            /* True if running in JSON mode, else
@@ -521,6 +520,11 @@ static void fossil_sqlite_log(void *notUsed, int iCode, const char *zErrmsg){
 #endif
   if( iCode==SQLITE_SCHEMA ) return;
   if( g.dbIgnoreErrors ) return;
+#ifdef SQLITE_READONLY_DIRECTORY
+  if( iCode==SQLITE_READONLY_DIRECTORY ){
+    zErrmsg = "database is in a read-only directory";
+  }
+#endif
   fossil_warning("%s: %s", fossil_sqlite_return_code_name(iCode), zErrmsg);
 }
 
@@ -555,6 +559,8 @@ int main(int argc, char **argv)
   const char *zCmdName = "unknown";
   const CmdOrPage *pCmd = 0;
   int rc;
+
+  fossil_limit_memory(1);
   if( sqlite3_libversion_number()<3014000 ){
     fossil_fatal("Unsuitable SQLite version %s, must be at least 3.14.0",
                  sqlite3_libversion());
@@ -622,7 +628,6 @@ int main(int argc, char **argv)
     const char *zChdir = find_option("chdir",0,1);
     g.isHTTP = 0;
     g.rcvid = 0;
-    g.fNoDirSymlinks = find_option("no-dir-symlinks", 0, 0)!=0;
     g.fQuiet = find_option("quiet", 0, 0)!=0;
     g.fSqlTrace = find_option("sqltrace", 0, 0)!=0;
     g.fSqlStats = find_option("sqlstats", 0, 0)!=0;
@@ -737,7 +742,7 @@ int main(int argc, char **argv)
   **
   ** TH_OK: The xFunc() and the TH1 notification will both be executed.
   **
-  ** TH_ERROR: The xFunc() will be executed, the TH1 notification will be
+  ** TH_ERROR: The xFunc() will be skipped, the TH1 notification will be
   **           skipped.  If the xFunc() is being hooked, the error message
   **           will be emitted.
   **
@@ -891,7 +896,7 @@ const char *find_repository_option(){
 void verify_all_options(void){
   int i;
   for(i=1; i<g.argc; i++){
-    if( g.argv[i][0]=='-' ){
+    if( g.argv[i][0]=='-' && g.argv[i][1]!=0 ){
       fossil_fatal(
         "unrecognized command-line option, or missing argument: %s",
         g.argv[i]);
@@ -951,8 +956,8 @@ static void get_version_blob(
 #if defined(FOSSIL_DEBUG)
   blob_append(pOut, "FOSSIL_DEBUG\n", -1);
 #endif
-#if defined(FOSSIL_OMIT_DELTA_CKSUM_TEST)
-  blob_append(pOut, "FOSSIL_OMIT_DELTA_CKSUM_TEST\n", -1);
+#if defined(FOSSIL_ENABLE_DELTA_CKSUM_TEST)
+  blob_append(pOut, "FOSSIL_ENABLE_DELTA_CKSUM_TEST\n", -1);
 #endif
 #if defined(FOSSIL_ENABLE_LEGACY_MV_RM)
   blob_append(pOut, "FOSSIL_ENABLE_LEGACY_MV_RM\n", -1);
@@ -996,6 +1001,9 @@ static void get_version_blob(
 #else
   blob_append(pOut, "FOSSIL_STATIC_BUILD\n", -1);
 #endif
+#if defined(HAVE_PLEDGE)
+  blob_append(pOut, "HAVE_PLEDGE\n", -1);
+#endif
 #if defined(USE_SEE)
   blob_append(pOut, "USE_SEE\n", -1);
 #endif
@@ -1063,7 +1071,7 @@ void test_version_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  verboseFlag = atoi(PD("verbose","0"));
+  verboseFlag = PD("verbose", 0) != 0;
   style_header("Version Information");
   style_submenu_element("Stat", "stat");
   get_version_blob(&versionInfo, verboseFlag);
@@ -1183,7 +1191,7 @@ static char *enter_chroot_jail(char *zRepo, int noJail){
     file_canonical_name(zRepo, &dir, 0);
     zDir = blob_str(&dir);
     if( !noJail ){
-      if( file_isdir(zDir)==1 ){
+      if( file_isdir(zDir, ExtFILE)==1 ){
         if( file_chdir(zDir, 1) ){
           fossil_fatal("unable to chroot into %s", zDir);
         }
@@ -1210,7 +1218,7 @@ static char *enter_chroot_jail(char *zRepo, int noJail){
     if(i){
       fossil_fatal("setgid/uid() failed with errno %d", errno);
     }
-    if( g.db==0 && file_isfile(zRepo) ){
+    if( g.db==0 && file_isfile(zRepo, ExtFILE) ){
       db_open_repository(zRepo);
     }
   }
@@ -1293,6 +1301,9 @@ static int repo_list_page(void){
         ** do not work for repositories whose names do not end in ".fossil".
         ** So do not hyperlink those cases. */
         @ <li>%h(zName)</li>
+      } else if( sqlite3_strglob("*/.*", zName)==0 ){
+        /* Do not show hidden repos */
+        @ <li>%h(zName) (hidden)</li>
       } else if( allRepo && sqlite3_strglob("[a-zA-Z]:/?*", zName)!=0 ){
         @ <li><a href="%R/%T(zUrl)/home" target="_blank">/%h(zName)</a></li>
       }else{
@@ -1426,7 +1437,7 @@ static void process_one_web_page(
       */
       zCleanRepo = file_cleanup_fullpath(zRepo);
       if( szFile==0 && sqlite3_strglob("*/.fossil",zRepo)!=0 ){
-        szFile = file_size(zCleanRepo);
+        szFile = file_size(zCleanRepo, ExtFILE);
         if( g.fHttpTrace ){
           char zBuf[24];
           sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", szFile);
@@ -1446,7 +1457,7 @@ static void process_one_web_page(
 
         /* The PATH_INFO prefix seen so far is a valid directory.
         ** Continue the loop with the next element of the PATH_INFO */
-        if( zPathInfo[i]=='/' && file_isdir(zCleanRepo)==1 ){
+        if( zPathInfo[i]=='/' && file_isdir(zCleanRepo, ExtFILE)==1 ){
           fossil_free(zToFree);
           i++;
           continue;
@@ -1464,14 +1475,14 @@ static void process_one_web_page(
         ** pages.
         */
         if( pFileGlob!=0
-         && file_isfile(zCleanRepo)
+         && file_isfile(zCleanRepo, ExtFILE)
          && glob_match(pFileGlob, file_cleanup_fullpath(zRepo))
          && sqlite3_strglob("*.fossil*",zRepo)!=0
          && (zMimetype = mimetype_from_name(zRepo))!=0
          && strcmp(zMimetype, "application/x-fossil-artifact")!=0
         ){
           Blob content;
-          blob_read_from_file(&content, file_cleanup_fullpath(zRepo));
+          blob_read_from_file(&content, file_cleanup_fullpath(zRepo), ExtFILE);
           cgi_set_content_type(zMimetype);
           cgi_set_content(&content);
           cgi_reply();
@@ -1541,8 +1552,29 @@ static void process_one_web_page(
   }
 
   /* At this point, the appropriate repository database file will have
-  ** been opened.  Use the first element of PATH_INFO as the page name
-  ** and deliver the appropriate page back to the user.
+  ** been opened.
+  **
+  ** Check to see if the the PATH_INFO begins with "draft[1-9]" and if
+  ** so activate the special handling for draft skins
+  */
+  if( zPathInfo && strncmp(zPathInfo,"/draft",6)==0
+   && zPathInfo[6]>='1' && zPathInfo[6]<='9'
+   && (zPathInfo[7]=='/' || zPathInfo[7]==0)
+  ){
+    int iSkin = zPathInfo[6] - '0';
+    char *zNewScript;
+    skin_use_draft(iSkin);
+    zNewScript = mprintf("%s/draft%d", P("SCRIPT_NAME"), iSkin);
+    if( g.zTop ) g.zTop = mprintf("%s/draft%d", g.zTop, iSkin);
+    if( g.zBaseURL ) g.zBaseURL = mprintf("%s/draft%d", g.zBaseURL, iSkin);
+    zPathInfo += 7;
+    cgi_replace_parameter("PATH_INFO", zPathInfo);
+    cgi_replace_parameter("SCRIPT_NAME", zNewScript);
+  }
+
+  /* If the content type is application/x-fossil or 
+  ** application/x-fossil-debug, then a sync/push/pull/clone is
+  ** desired, so default the PATH_INFO to /xfer
   */
   if( g.zContentType &&
       strncmp(g.zContentType, "application/x-fossil", 20)==0 ){
@@ -1551,6 +1583,10 @@ static void process_one_web_page(
     ** invoke the sync page. */
     zPathInfo = "/xfer";
   }
+
+  /* Use the first element of PATH_INFO as the page name
+  ** and deliver the appropriate page back to the user.
+  */
   set_base_url(0);
   if( zPathInfo==0 || zPathInfo[0]==0
       || (zPathInfo[0]=='/' && zPathInfo[1]==0) ){
@@ -1618,7 +1654,9 @@ static void process_one_web_page(
   /* Locate the method specified by the path and execute the function
   ** that implements that method.
   */
-  if( dispatch_name_search(g.zPath-1, CMDFLAG_WEBPAGE, &pCmd) ){
+  if( dispatch_name_search(g.zPath-1, CMDFLAG_WEBPAGE, &pCmd)
+   && dispatch_alias(g.zPath-1, &pCmd)
+  ){
 #ifdef FOSSIL_ENABLE_JSON
     if(g.json.isJsonMode){
       json_err(FSL_JSON_E_RESOURCE_NOT_FOUND,NULL,0);
@@ -1664,7 +1702,7 @@ static void process_one_web_page(
     **
     ** TH_OK: The xFunc() and the TH1 notification will both be executed.
     **
-    ** TH_ERROR: The xFunc() will be executed, the TH1 notification will be
+    ** TH_ERROR: The xFunc() will be skipped, the TH1 notification will be
     **           skipped.  If the xFunc() is being hooked, the error message
     **           will be emitted.
     **
@@ -1841,7 +1879,7 @@ void cmd_cgi(void){
   fossil_binary_mode(g.httpOut);
   fossil_binary_mode(g.httpIn);
   g.cgiOutput = 1;
-  blob_read_from_file(&config, zFile);
+  blob_read_from_file(&config, zFile, ExtFILE);
   while( blob_line(&config, &line) ){
     if( !blob_token(&line, &key) ) continue;
     if( blob_buffer(&key)[0]=='#' ) continue;
@@ -1998,7 +2036,7 @@ void cmd_cgi(void){
 ** contains multiple repositories that can be served.  If g.argv[arg]
 ** is a directory, the repositories it contains must be named
 ** "*.fossil".  If g.argv[arg] does not exist, then we must be within
-** an open check-out and the repository serve is the repository of
+** an open check-out and the repository to serve is the repository of
 ** that check-out.
 **
 ** Open the repository to be served if it is known.  If g.argv[arg] is
@@ -2015,7 +2053,7 @@ static void find_server_repository(int arg, int fCreate){
     db_must_be_within_tree();
   }else{
     const char *zRepo = g.argv[arg];
-    int isDir = file_isdir(zRepo);
+    int isDir = file_isdir(zRepo, ExtFILE);
     if( isDir==1 ){
       g.zRepositoryName = mprintf("%s", zRepo);
       file_simplify_name(g.zRepositoryName, -1, 0);
@@ -2291,6 +2329,15 @@ static int binaryOnPath(const char *zBinary){
 #endif
 #endif
 
+/*
+** Send a time-out reply
+*/
+void sigalrm_handler(int x){
+  printf("TIMEOUT\n");
+  fflush(stdout);
+  exit(1);
+}
+
 /*
 ** COMMAND: server*
 ** COMMAND: ui
@@ -2342,6 +2389,8 @@ static int binaryOnPath(const char *zBinary){
 **   --localauth         enable automatic login for requests from localhost
 **   --localhost         listen on 127.0.0.1 only (always true for "ui")
 **   --https             signal a request coming in via https
+**   --max-latency N     Do not let any single HTTP request run for more than N
+**                       seconds (only works on unix)
 **   --nojail            Drop root privileges but do not enter the chroot jail
 **   --nossl             signal that no SSL connections are available
 **   --notfound URL      Redirect
@@ -2365,6 +2414,7 @@ void cmd_webserver(void){
   int flags = 0;            /* Server flags */
 #if !defined(_WIN32)
   int noJail;               /* Do not enter the chroot jail */
+  const char *zMaxLatency;   /* Maximum runtime of any single HTTP request */
 #endif
   int allowRepoList;         /* List repositories on URL "/" */
   const char *zAltBase;      /* Argument to the --baseurl option */
@@ -2392,6 +2442,7 @@ void cmd_webserver(void){
   skin_override();
 #if !defined(_WIN32)
   noJail = find_option("nojail",0,0)!=0;
+  zMaxLatency = find_option("max-latency",0,1);
 #endif
   g.useLocalauth = find_option("localauth", 0, 0)!=0;
   Th_InitTraceLog();
@@ -2448,11 +2499,17 @@ void cmd_webserver(void){
     }
   }
   if( zPort ){
-    int i;
-    for(i=strlen(zPort)-1; i>=0 && zPort[i]!=':'; i--){}
-    if( i>0 ){
-      zIpAddr = mprintf("%.*s", i, zPort);
-      zPort += i+1;
+    if( strchr(zPort,':') ){
+      int i;
+      for(i=strlen(zPort)-1; i>=0 && zPort[i]!=':'; i--){}
+      if( i>0 ){
+        if( zPort[0]=='[' && zPort[i-1]==']' ){
+          zIpAddr = mprintf("%.*s", i-2, zPort+1);
+        }else{
+          zIpAddr = mprintf("%.*s", i, zPort);
+        }
+        zPort += i+1;
+      }
     }
     iPort = mxPort = atoi(zPort);
   }else{
@@ -2479,12 +2536,15 @@ void cmd_webserver(void){
 #else
     zBrowser = db_get("web-browser", "open");
 #endif
-    if( zIpAddr ){
-      zBrowserCmd = mprintf("%s http://%s:%%d/%s &",
-                            zBrowser, zIpAddr, zInitPage);
-    }else{
+    if( zIpAddr==0 ){
       zBrowserCmd = mprintf("%s http://localhost:%%d/%s &",
                             zBrowser, zInitPage);
+    }else if( strchr(zIpAddr,':') ){
+      zBrowserCmd = mprintf("%s http://[%s]:%%d/%s &",
+                            zBrowser, zIpAddr, zInitPage);
+    }else{
+      zBrowserCmd = mprintf("%s http://%s:%%d/%s &",
+                            zBrowser, zIpAddr, zInitPage);
     }
   }
   if( g.repositoryOpen ) flags |= HTTP_SERVER_HAD_REPOSITORY;
@@ -2493,6 +2553,10 @@ void cmd_webserver(void){
   if( cgi_http_server(iPort, mxPort, zBrowserCmd, zIpAddr, flags) ){
     fossil_fatal("unable to listen on TCP socket %d", iPort);
   }
+  if( zMaxLatency ){
+    signal(SIGALRM, sigalrm_handler);
+    alarm(atoi(zMaxLatency));
+  }
   g.httpIn = stdin;
   g.httpOut = stdout;
   if( g.fHttpTrace || g.fSqlTrace ){
@@ -2515,12 +2579,15 @@ void cmd_webserver(void){
   /* Win32 implementation */
   if( isUiCmd ){
     zBrowser = db_get("web-browser", "start");
-    if( zIpAddr ){
-      zBrowserCmd = mprintf("%s http://%s:%%d/%s &",
-                            zBrowser, zIpAddr, zInitPage);
-    }else{
+    if( zIpAddr==0 ){
       zBrowserCmd = mprintf("%s http://localhost:%%d/%s &",
                             zBrowser, zInitPage);
+    }else if( strchr(zIpAddr,':') ){
+      zBrowserCmd = mprintf("%s http://[%s]:%%d/%s &",
+                            zBrowser, zIpAddr, zInitPage);
+    }else{
+      zBrowserCmd = mprintf("%s http://%s:%%d/%s &",
+                            zBrowser, zIpAddr, zInitPage);
     }
   }
   if( g.repositoryOpen ) flags |= HTTP_SERVER_HAD_REPOSITORY;
