@@ -589,7 +589,7 @@ tag_cmd_usage:
 /*
 ** COMMAND: reparent*
 **
-** Usage: %fossil reparent [OPTIONS] CHECK-IN PARENT ....
+** Usage: %fossil reparent [OPTIONS] CHECK-IN PARENT ...
 **
 ** Create a "parent" tag that causes CHECK-IN to be interpreted as a
 ** child of PARENT.  If multiple PARENTs are listed, then the first is
@@ -621,7 +621,7 @@ void reparent_cmd(void){
   db_find_and_open_repository(0, 0);
   verify_all_options();
   if( g.argc<4 ){
-    usage("reparent [OPTIONS] PARENT ...");
+    usage("[OPTIONS] CHECK-IN PARENT ...");
   }
   rid = name_to_typed_rid(g.argv[2], "ci");
   blob_init(&value, 0, 0);
@@ -671,7 +671,7 @@ void taglist_page(void){
   while( db_step(&q)==SQLITE_ROW ){
     const char *zName = db_column_text(&q, 0);
     if( g.perm.Hyperlink ){
-      @ <li>%z(xhref("class='taglink'","%R/timeline?t=%T&n=200",zName))
+      @ <li>%z(chref("taglink","%R/timeline?t=%T&n=200",zName))
       @ %h(zName)</a></li>
     }else{
       @ <li><span class="tagDsp">%h(zName)</span></li>
