@@ -426,20 +426,26 @@ static void tar_finish(Blob *pOut){
 **
 ** Generate a GZIP-compressed tarball in the file given by the first argument
 ** that contains files given in the second and subsequent arguments.
+**
+**   -h, --dereference   Follow symlinks; archive the files they point to.
 */
 void test_tarball_cmd(void){
   int i;
   Blob zip;
+  int eFType = SymFILE;
   if( g.argc<3 ){
-    usage("ARCHIVE FILE....");
+    usage("ARCHIVE [options] FILE....");
+  }
+  if( find_option("dereference","h",0) ){
+    eFType = ExtFILE;
   }
   sqlite3_open(":memory:", &g.db);
   tar_begin(-1);
   for(i=3; i<g.argc; i++){
     Blob file;
     blob_zero(&file);
-    blob_read_from_file(&file, g.argv[i]);
-    tar_add_file(g.argv[i], &file, file_wd_perm(0), file_wd_mtime(0));
+    blob_read_from_file(&file, g.argv[i], eFType);
+    tar_add_file(g.argv[i], &file, file_perm(0,0), file_mtime(0,0));
     blob_reset(&file);
   }
   tar_finish(&zip);
@@ -618,6 +624,7 @@ void tarball_cmd(void){
   if( g.argc!=4 ){
     usage("VERSION OUTPUTFILE");
   }
+  g.zOpenRevision = g.argv[2];
   rid = name_to_typed_rid(g.argv[2], "ci");
   if( rid==0 ){
     fossil_fatal("Check-in not found: %s", g.argv[2]);
@@ -691,7 +698,7 @@ void tarball_page(void){
   z = P("r");
   if( z==0 ) z = P("uuid");
   if( z==0 ) z = "trunk";
-  zRid = fossil_strdup(z);
+  g.zOpenRevision = zRid = fossil_strdup(z);
   nRid = strlen(zRid);
   zInclude = P("in");
   if( zInclude ) pInclude = glob_create(zInclude);
@@ -711,8 +718,9 @@ void tarball_page(void){
       }
     }
   }
-  rid = name_to_typed_rid(nRid?zRid:zName, "ci");
+  rid = symbolic_name_to_rid(nRid?zRid:zName, "ci");
   if( rid==0 ){
+    cgi_set_status(404, "Not Found");
     @ Not found
     return;
   }
