@@ -240,7 +240,8 @@ static int contains_whitespace(const char *zName){
 **                            -v|--verbose     Extra diagnostic output
 **                            -n|--dryrun      Show what would have happened
 **
-**    remove | rm FILE ... Remove unversioned files from the local repository.
+**    remove|rm|delete FILE ...
+**                         Remove unversioned files from the local repository.
 **                         Changes are not pushed to other repositories until
 **                         the next sync.
 **
@@ -276,6 +277,7 @@ void unversioned_cmd(void){
     if( mtime<=0 ) fossil_fatal("bad timestamp: %Q", zMtime);
   }
   if( memcmp(zCmd, "add", nCmd)==0 ){
+    const char *zError = 0;
     const char *zIn;
     const char *zAs;
     Blob file;
@@ -288,14 +290,20 @@ void unversioned_cmd(void){
     content_rcvid_init("#!fossil unversioned add");
     for(i=3; i<g.argc; i++){
       zIn = zAs ? zAs : g.argv[i];
-      if( zIn[0]==0 || zIn[0]=='/' || !file_is_simple_pathname(zIn,1) ){
-        fossil_fatal("'%Q' is not an acceptable filename", zIn);
+      if( zIn[0]==0 ){
+        zError = "be empty string";
+      }else if( zIn[0]=='/' ){
+        zError = "be absolute";
+      }else if ( !file_is_simple_pathname(zIn,1) ){
+        zError = "contain complex paths";
+      }else if( contains_whitespace(zIn) ){
+        zError = "contain whitespace";
       }
-      if( contains_whitespace(zIn) ){
-        fossil_fatal("names of unversioned files may not contain whitespace");
+      if( zError ){
+        fossil_fatal("unversioned filenames may not %s: %Q", zError, zIn);
       }
       blob_init(&file,0,0);
-      blob_read_from_file(&file, g.argv[i]);
+      blob_read_from_file(&file, g.argv[i], ExtFILE);
       unversioned_write(zIn, &file, mtime);
       blob_reset(&file);
     }
@@ -344,7 +352,7 @@ void unversioned_cmd(void){
     }
     fossil_free(zCmd);
     blob_reset(&content);
-    blob_read_from_file(&content, zTFile);
+    blob_read_from_file(&content, zTFile, ExtFILE);
 #if defined(_WIN32) || defined(__CYGWIN__)
     blob_to_lf_only(&content);
 #endif
@@ -412,7 +420,8 @@ void unversioned_cmd(void){
     g.argv[1] = "sync";
     g.argv[2] = "--uv-noop";
     sync_unversioned(syncFlags);
-  }else if( memcmp(zCmd, "remove", nCmd)==0 || memcmp(zCmd, "rm", nCmd)==0 ){
+  }else if( memcmp(zCmd, "remove", nCmd)==0 || memcmp(zCmd, "rm", nCmd)==0
+         || memcmp(zCmd, "delete", nCmd)==0 ){
     int i;
     verify_all_options();
     db_begin_transaction();
@@ -454,14 +463,16 @@ void unversioned_cmd(void){
 ** Query parameters:
 **
 **    byage=1          Order the initial display be decreasing age
+**    showdel=0        Show deleted files
 */
-void uvstat_page(void){
+void uvlist_page(void){
   Stmt q;
   sqlite3_int64 iNow;
   sqlite3_int64 iTotalSz = 0;
   int cnt = 0;
   int n = 0;
   const char *zOrderBy = "name";
+  int showDel = 0;
   char zSzName[100];
 
   login_check_credentials();
@@ -473,6 +484,117 @@ void uvstat_page(void){
     return;
   }
   if( PB("byage") ) zOrderBy = "mtime DESC";
+  if( PB("showdel") ) showDel = 1;
+  db_prepare(&q,
+    "SELECT"
+    "   name,"
+    "   mtime,"
+    "   hash,"
+    "   sz,"
+    "   (SELECT login FROM rcvfrom, user"
+    "     WHERE user.uid=rcvfrom.uid AND rcvfrom.rcvid=unversioned.rcvid),"
+    "   rcvid"
+    " FROM unversioned %s ORDER BY %s",
+    showDel ? "" : "WHERE hash IS NOT NULL" /*safe-for-%s*/,
+    zOrderBy/*safe-for-%s*/
+  );
+  iNow = db_int64(0, "SELECT strftime('%%s','now');");
+  while( db_step(&q)==SQLITE_ROW ){
+    const char *zName = db_column_text(&q, 0);
+    sqlite3_int64 mtime = db_column_int(&q, 1);
+    const char *zHash = db_column_text(&q, 2);
+    int isDeleted = zHash==0;
+    int fullSize = db_column_int(&q, 3);
+    char *zAge = human_readable_age((iNow - mtime)/86400.0);
+    const char *zLogin = db_column_text(&q, 4);
+    int rcvid = db_column_int(&q,5);
+    if( zLogin==0 ) zLogin = "";
+    if( (n++)==0 ){
+      style_table_sorter();
+      @ <div class="uvlist">
+      @ <table cellpadding="2" cellspacing="0" border="1" class='sortable' \
+      @  data-column-types='tkKttn' data-init-sort='1'>
+      @ <thead><tr>
+      @   <th> Name
+      @   <th> Age
+      @   <th> Size
+      @   <th> User
+      @   <th> SHA1
+      if( g.perm.Admin ){
+        @ <th> rcvid
+      }
+      @ </tr></thead>
+      @ <tbody>
+    }
+    @ <tr>
+    if( isDeleted ){
+      sqlite3_snprintf(sizeof(zSzName), zSzName, "<i>Deleted</i>");
+      zHash = "";
+      fullSize = 0;
+      @ <td> %h(zName) </td>
+    }else{
+      approxSizeName(sizeof(zSzName), zSzName, fullSize);
+      iTotalSz += fullSize;
+      cnt++;
+      @ <td> <a href='%R/uv/%T(zName)'>%h(zName)</a> </td>
+    }
+    @ <td data-sortkey='%016llx(-mtime)'> %s(zAge) </td>
+    @ <td data-sortkey='%08x(fullSize)'> %s(zSzName) </td>
+    @ <td> %h(zLogin) </td>
+    @ <td> %h(zHash) </td>
+    if( g.perm.Admin ){
+      if( rcvid ){
+        @ <td> <a href="%R/rcvfrom?rcvid=%d(rcvid)">%d(rcvid)</a>
+      }else{
+        @ <td>
+      }
+    }
+    @ </tr>
+    fossil_free(zAge);
+  }
+  db_finalize(&q);
+  if( n ){
+    approxSizeName(sizeof(zSzName), zSzName, iTotalSz);
+    @ </tbody>
+    @ <tfoot><tr><td><b>Total over %d(cnt) files</b><td><td>%s(zSzName)
+    @ <td><td>
+    if( g.perm.Admin ){
+      @ <td>
+    }
+    @ </tfoot>
+    @ </table></div>
+  }else{
+    @ No unversioned files on this server.
+  }
+  style_footer();
+}
+
+/*
+** WEBPAGE: juvlist
+**
+** Return a complete list of unversioned files as JSON.  The JSON
+** looks like this:
+**
+** [{"name":NAME,
+**   "mtime":MTIME,
+**   "hash":HASH,
+**   "size":SIZE,
+**   "user":USER}]
+*/
+void uvlist_json_page(void){
+  Stmt q;
+  char *zSep = "[";
+  Blob json;
+
+  login_check_credentials();
+  if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  cgi_set_content_type("text/json");
+  if( !db_table_exists("repository","unversioned") ){
+    blob_init(&json, "[]", -1);
+    cgi_set_content(&json);
+    return;
+  }
+  blob_init(&json, 0, 0);
   db_prepare(&q,
      "SELECT"
      "   name,"
@@ -480,73 +602,26 @@ void uvstat_page(void){
      "   hash,"
      "   sz,"
      "   (SELECT login FROM rcvfrom, user"
-     "     WHERE user.uid=rcvfrom.uid AND rcvfrom.rcvid=unversioned.rcvid),"
-     "   rcvid"
-     " FROM unversioned ORDER BY %s",
-     zOrderBy/*safe-for-%s*/
+     "     WHERE user.uid=rcvfrom.uid AND rcvfrom.rcvid=unversioned.rcvid)"
+     " FROM unversioned WHERE hash IS NOT NULL"
    );
-   iNow = db_int64(0, "SELECT strftime('%%s','now');");
    while( db_step(&q)==SQLITE_ROW ){
      const char *zName = db_column_text(&q, 0);
      sqlite3_int64 mtime = db_column_int(&q, 1);
      const char *zHash = db_column_text(&q, 2);
-     int isDeleted = zHash==0;
      int fullSize = db_column_int(&q, 3);
-     char *zAge = human_readable_age((iNow - mtime)/86400.0);
      const char *zLogin = db_column_text(&q, 4);
-     int rcvid = db_column_int(&q,5);
      if( zLogin==0 ) zLogin = "";
-     if( (n++)==0 ){
-       @ <div class="uvlist">
-       @ <table cellpadding="2" cellspacing="0" border="1" id="uvtab">
-       @ <thead><tr>
-       @   <th> Name
-       @   <th> Age
-       @   <th> Size
-       @   <th> User
-       @   <th> SHA1
-       if( g.perm.Admin ){
-         @ <th> rcvid
-       }
-       @ </tr></thead>
-       @ <tbody>
-     }
-     @ <tr>
-     if( isDeleted ){
-       sqlite3_snprintf(sizeof(zSzName), zSzName, "<i>Deleted</i>");
-       zHash = "";
-       fullSize = 0;
-       @ <td> %h(zName) </td>
-     }else{
-       approxSizeName(sizeof(zSzName), zSzName, fullSize);
-       iTotalSz += fullSize;
-       cnt++;
-       @ <td> <a href='%R/uv/%T(zName)'>%h(zName)</a> </td>
-     }
-     @ <td data-sortkey='%016llx(-mtime)'> %s(zAge) </td>
-     @ <td data-sortkey='%08x(fullSize)'> %s(zSzName) </td>
-     @ <td> %h(zLogin) </td>
-     @ <td> %h(zHash) </td>
-     if( g.perm.Admin ){
-       if( rcvid ){
-         @ <td> <a href="%R/rcvfrom?rcvid=%d(rcvid)">%d(rcvid)</a>
-       }else{
-         @ <td>
-       }
-     }
-     @ </tr>
-     fossil_free(zAge);
+     blob_appendf(&json, "%s{\"name\":\"", zSep);
+     zSep = ",\n ";
+     blob_append_json_string(&json, zName);
+     blob_appendf(&json, "\",\n  \"mtime\":%lld,\n  \"hash\":\"", mtime);
+     blob_append_json_string(&json, zHash);
+     blob_appendf(&json, "\",\n  \"size\":%d,\n  \"user\":\"", fullSize);
+     blob_append_json_string(&json, zLogin);
+     blob_appendf(&json, "\"}");
    }
    db_finalize(&q);
-   if( n ){
-     approxSizeName(sizeof(zSzName), zSzName, iTotalSz);
-     @ </tbody>
-     @ <tfoot><tr><td><b>Total over %d(cnt) files</b><td><td>%s(zSzName)
-     @ <td><td></tfoot>
-     @ </table></div>
-     output_table_sorting_javascript("uvtab","tkKttN",1);
-   }else{
-     @ No unversioned files on this server.
-   }
-   style_footer();
+   blob_appendf(&json,"]\n");
+   cgi_set_content(&json);
 }
