@@ -15,7 +15,7 @@
 **
 *******************************************************************************
 **
-** This file contains code used to generate ZIP archives.
+** This file contains code used to generate ZIP and SQLAR archives.
 */
 #include "config.h"
 #include <assert.h>
@@ -27,6 +27,12 @@
 #endif
 #include "zip.h"
 
+/*
+** Type of archive to build.
+*/
+#define ARCHIVE_ZIP   0
+#define ARCHIVE_SQLAR 1
+
 /*
 ** Write a 16- or 32-bit integer as little-endian into the given buffer.
 */
@@ -53,6 +59,154 @@ static int unixTime; /* Seconds since 1970 */
 static int nDir;     /* Number of entries in azDir[] */
 static char **azDir; /* Directory names already added to the archive */
 
+typedef struct Archive Archive;
+struct Archive {
+  int eType;                      /* Type of archive (SQLAR or ZIP) */
+  Blob *pBlob;                    /* Output blob */
+  Blob tmp;                       /* Blob used as temp space for compression */
+  sqlite3 *db;                    /* Db used to assemble sqlar archive */
+  sqlite3_stmt *pInsert;          /* INSERT statement for SQLAR */
+  sqlite3_vfs vfs;                /* VFS object */
+};
+
+/*
+** Ensure that blob pBlob is at least nMin bytes in size.
+*/
+static void zip_blob_minsize(Blob *pBlob, int nMin){
+  if( blob_size(pBlob)<nMin ){
+    blob_resize(pBlob, nMin);
+  }
+}
+
+/*************************************************************************
+** Implementation of "archive" VFS. A VFS designed to store the contents
+** of a new database in a Blob. Used to construct sqlar archives in
+** memory.
+*/
+typedef struct ArchiveFile ArchiveFile;
+struct ArchiveFile {
+  sqlite3_file base;              /* Base class */
+  Blob *pBlob;
+};
+
+static int archiveClose(sqlite3_file *pFile){
+  return SQLITE_OK;
+}
+static int archiveRead(
+    sqlite3_file *pFile, void *pBuf, int iAmt, sqlite3_int64 iOfst
+){
+  assert( iOfst==0 || iOfst==24 );
+  return SQLITE_IOERR_SHORT_READ;
+}
+static int archiveWrite(
+    sqlite3_file *pFile, const void *pBuf, int iAmt, sqlite3_int64 iOfst
+){
+  ArchiveFile *pAF = (ArchiveFile*)pFile;
+  int nMin = (int)iOfst + iAmt;
+  char *aBlob;                    /* Output buffer */
+
+  zip_blob_minsize(pAF->pBlob, nMin);
+  aBlob = blob_buffer(pAF->pBlob);
+  memcpy(&aBlob[iOfst], pBuf, iAmt);
+  return SQLITE_OK;
+}
+static int archiveTruncate(sqlite3_file *pFile, sqlite3_int64 size){
+  return SQLITE_OK;
+}
+static int archiveSync(sqlite3_file *pFile, int flags){
+  return SQLITE_OK;
+}
+static int archiveFileSize(sqlite3_file *pFile, sqlite3_int64 *pSize){
+  *pSize = 0;
+  return SQLITE_OK;
+}
+static int archiveLock(sqlite3_file *pFile, int eLock){
+  return SQLITE_OK;
+}
+static int archiveUnlock(sqlite3_file *pFile, int eLock){
+  return SQLITE_OK;
+}
+static int archiveCheckReservedLock(sqlite3_file *pFile, int *pResOut){
+  *pResOut = 0;
+  return SQLITE_OK;
+}
+static int archiveFileControl(sqlite3_file *pFile, int op, void *pArg){
+  if( op==SQLITE_FCNTL_SIZE_HINT ){
+    ArchiveFile *pAF = (ArchiveFile*)pFile;
+    zip_blob_minsize(pAF->pBlob, (int)(*(sqlite3_int64*)pArg));
+  }
+  return SQLITE_NOTFOUND;
+}
+static int archiveSectorSize(sqlite3_file *pFile){
+  return 512;
+}
+static int archiveDeviceCharacteristics(sqlite3_file *pFile){
+  return 0;
+}
+
+static int archiveOpen(
+  sqlite3_vfs *pVfs, const char *zName, 
+  sqlite3_file *pFile, int flags, int *pOutFlags
+){
+  static struct sqlite3_io_methods methods = {
+    1,             /* iVersion */
+    archiveClose,
+    archiveRead,
+    archiveWrite,
+    archiveTruncate,
+    archiveSync,
+    archiveFileSize,
+    archiveLock,
+    archiveUnlock,
+    archiveCheckReservedLock,
+    archiveFileControl,
+    archiveSectorSize,
+    archiveDeviceCharacteristics,
+    0, 0, 0, 0,
+    0, 0
+  };
+
+  ArchiveFile *pAF = (ArchiveFile*)pFile;
+  assert( flags & SQLITE_OPEN_MAIN_DB );
+
+  pAF->base.pMethods = &methods;
+  pAF->pBlob = (Blob*)pVfs->pAppData;
+
+  return SQLITE_OK;
+}
+static int archiveDelete(sqlite3_vfs *pVfs, const char *zName, int syncDir){
+  return SQLITE_OK;
+}
+static int archiveAccess(
+    sqlite3_vfs *pVfs, const char *zName, int flags, int *pResOut
+){
+  *pResOut = 0;
+  return SQLITE_OK;
+}
+static int archiveFullPathname(
+  sqlite3_vfs *pVfs, const char *zIn, int nOut, char *zOut
+){
+  int n = strlen(zIn);
+  memcpy(zOut, zIn, n+1);
+  return SQLITE_OK;
+}
+static int archiveRandomness(sqlite3_vfs *pVfs, int nByte, char *zOut){
+  memset(zOut, 0, nByte);
+  return SQLITE_OK;
+}
+static int archiveSleep(sqlite3_vfs *pVfs, int microseconds){
+  return SQLITE_OK;
+}
+static int archiveCurrentTime(sqlite3_vfs *pVfs, double *prOut){
+  return SQLITE_OK;
+}
+static int archiveGetLastError(sqlite3_vfs *pVfs, int nBuf, char *aBuf){
+  return SQLITE_OK;
+}
+/*
+** End of "archive" VFS.
+*************************************************************************/
+
 /*
 ** Initialize a new ZIP archive.
 */
@@ -92,39 +246,18 @@ void zip_set_timedate(double rDate){
   unixTime = (rDate - 2440587.5)*86400.0;
 }
 
-/*
-** If the given filename includes one or more directory entries, make
-** sure the directories are already in the archive.  If they are not
-** in the archive, add them.
-*/
-void zip_add_folders(char *zName){
-  int i, c;
-  int j;
-  for(i=0; zName[i]; i++){
-    if( zName[i]=='/' ){
-      c = zName[i+1];
-      zName[i+1] = 0;
-      for(j=0; j<nDir; j++){
-        if( fossil_strcmp(zName, azDir[j])==0 ) break;
-      }
-      if( j>=nDir ){
-        nDir++;
-        azDir = fossil_realloc(azDir, sizeof(azDir[0])*nDir);
-        azDir[j] = mprintf("%s", zName);
-        zip_add_file(zName, 0, 0);
-      }
-      zName[i+1] = c;
-    }
-  }
-}
-
 /*
 ** Append a single file to a growing ZIP archive.
 **
 ** pFile is the file to be appended.  zName is the name
 ** that the file should be saved as.
 */
-void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
+static void zip_add_file_to_zip(
+  Archive *p,
+  const char *zName, 
+  const Blob *pFile, 
+  int mPerm
+){
   z_stream stream;
   int nameLen;
   int toOut = 0;
@@ -143,6 +276,8 @@ void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
 
   /* Fill in as much of the header as we know.
   */
+  nameLen = (int)strlen(zName);
+  if( nameLen==0 ) return;
   nBlob = pFile ? blob_size(pFile) : 0;
   if( pFile ){ /* This is a file, possibly empty... */
     iMethod = (nBlob>0) ? 8 : 0; /* Cannot compress zero bytes. */
@@ -155,7 +290,6 @@ void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
     iMethod = 0;
     iMode = 040755;
   }
-  nameLen = strlen(zName);
   memset(zHdr, 0, sizeof(zHdr));
   put32(&zHdr[0], 0x04034b50);
   put16(&zHdr[4], 0x000a);
@@ -245,33 +379,187 @@ void zip_add_file(const char *zName, const Blob *pFile, int mPerm){
   nEntry++;
 }
 
+static void zip_add_file_to_sqlar(
+  Archive *p,
+  const char *zName, 
+  const Blob *pFile, 
+  int mPerm
+){
+  int nName = (int)strlen(zName);
+
+  if( p->db==0 ){
+    assert( p->vfs.zName==0 );
+    p->vfs.zName = (const char*)mprintf("archivevfs%p", (void*)p);
+    p->vfs.iVersion = 1;
+    p->vfs.szOsFile = sizeof(ArchiveFile);
+    p->vfs.mxPathname = 512;
+    p->vfs.pAppData = (void*)p->pBlob;
+    p->vfs.xOpen = archiveOpen;
+    p->vfs.xDelete = archiveDelete;
+    p->vfs.xAccess = archiveAccess;
+    p->vfs.xFullPathname = archiveFullPathname;
+    p->vfs.xRandomness = archiveRandomness;
+    p->vfs.xSleep = archiveSleep;
+    p->vfs.xCurrentTime = archiveCurrentTime;
+    p->vfs.xGetLastError = archiveGetLastError;
+    sqlite3_vfs_register(&p->vfs, 0);
+    sqlite3_open_v2("file:xyz.db", &p->db, 
+        SQLITE_OPEN_CREATE|SQLITE_OPEN_READWRITE, p->vfs.zName
+    );
+    assert( p->db );
+    blob_zero(&p->tmp);
+    sqlite3_exec(p->db, 
+        "PRAGMA page_size=512;"
+        "PRAGMA journal_mode = off;"
+        "PRAGMA cache_spill = off;"
+        "BEGIN;"
+        "CREATE TABLE sqlar("
+          "name TEXT PRIMARY KEY,  -- name of the file\n"
+          "mode INT,               -- access permissions\n"
+          "mtime INT,              -- last modification time\n"
+          "sz INT,                 -- original file size\n"
+          "data BLOB               -- compressed content\n"
+        ");", 0, 0, 0
+    );
+    sqlite3_prepare(p->db, 
+        "INSERT INTO sqlar VALUES(?, ?, ?, ?, ?)", -1, 
+        &p->pInsert, 0
+    );
+    assert( p->pInsert );
+
+    sqlite3_bind_int64(p->pInsert, 3, unixTime);
+    blob_zero(p->pBlob);
+  }
+
+  if( nName==0 ) return;
+  if( pFile==0 ){
+    /* Directory. */
+    if( zName[nName-1]=='/' ) nName--;
+    sqlite3_bind_text(p->pInsert, 1, zName, nName, SQLITE_STATIC);
+    sqlite3_bind_int(p->pInsert, 2, 040755);
+    sqlite3_bind_int(p->pInsert, 4, 0);
+    sqlite3_bind_null(p->pInsert, 5);
+  }else{
+    sqlite3_bind_text(p->pInsert, 1, zName, nName, SQLITE_STATIC);
+    if( mPerm==PERM_LNK ){
+      sqlite3_bind_int(p->pInsert, 2, 0120755);
+      sqlite3_bind_int(p->pInsert, 4, -1);
+      sqlite3_bind_text(p->pInsert, 5, 
+          blob_buffer(pFile), blob_size(pFile), SQLITE_STATIC
+      );
+    }else{
+      int nIn = blob_size(pFile);
+      unsigned long int nOut = nIn;
+      sqlite3_bind_int(p->pInsert, 2, mPerm==PERM_EXE ? 0100755 : 0100644);
+      sqlite3_bind_int(p->pInsert, 4, nIn);
+      zip_blob_minsize(&p->tmp, nIn);
+      compress( (unsigned char*)
+          blob_buffer(&p->tmp), &nOut, (unsigned char*)blob_buffer(pFile), nIn
+      );
+      if( nOut>=nIn ){
+        sqlite3_bind_blob(p->pInsert, 5, 
+            blob_buffer(pFile), blob_size(pFile), SQLITE_STATIC
+        );
+      }else{
+        sqlite3_bind_blob(p->pInsert, 5, 
+            blob_buffer(&p->tmp), nOut, SQLITE_STATIC
+        );
+      }
+    }
+  }
+
+  sqlite3_step(p->pInsert);
+  sqlite3_reset(p->pInsert);
+}
+
+static void zip_add_file(
+  Archive *p,
+  const char *zName, 
+  const Blob *pFile, 
+  int mPerm
+){
+  if( p->eType==ARCHIVE_ZIP ){
+    zip_add_file_to_zip(p, zName, pFile, mPerm);
+  }else{
+    zip_add_file_to_sqlar(p, zName, pFile, mPerm);
+  }
+}
+
+/*
+** If the given filename includes one or more directory entries, make
+** sure the directories are already in the archive.  If they are not
+** in the archive, add them.
+*/
+static void zip_add_folders(Archive *p, char *zName){
+  int i, c;
+  int j;
+  for(i=0; zName[i]; i++){
+    if( zName[i]=='/' ){
+      c = zName[i+1];
+      zName[i+1] = 0;
+      for(j=0; j<nDir; j++){
+        if( fossil_strcmp(zName, azDir[j])==0 ) break;
+      }
+      if( j>=nDir ){
+        nDir++;
+        azDir = fossil_realloc(azDir, sizeof(azDir[0])*nDir);
+        azDir[j] = mprintf("%s", zName);
+        zip_add_file(p, zName, 0, 0);
+      }
+      zName[i+1] = c;
+    }
+  }
+}
+
+/*
+** Free all the members of structure Archive allocated while processing
+** an SQLAR request.
+*/
+static void free_archive(Archive *p){
+  if( p->vfs.zName ){
+    sqlite3_vfs_unregister(&p->vfs);
+    fossil_free((char*)p->vfs.zName);
+    p->vfs.zName = 0;
+  }
+  sqlite3_finalize(p->pInsert);
+  p->pInsert = 0;
+  sqlite3_close(p->db);
+  p->db = 0;
+}
 
 /*
 ** Write the ZIP archive into the given BLOB.
 */
-void zip_close(Blob *pZip){
-  int iTocStart;
-  int iTocEnd;
+static void zip_close(Archive *p){
   int i;
-  char zBuf[30];
+  if( p->eType==ARCHIVE_ZIP ){
+    int iTocStart;
+    int iTocEnd;
+    char zBuf[30];
 
-  iTocStart = blob_size(&body);
-  blob_append(&body, blob_buffer(&toc), blob_size(&toc));
-  iTocEnd = blob_size(&body);
+    iTocStart = blob_size(&body);
+    blob_append(&body, blob_buffer(&toc), blob_size(&toc));
+    iTocEnd = blob_size(&body);
+
+    memset(zBuf, 0, sizeof(zBuf));
+    put32(&zBuf[0], 0x06054b50);
+    put16(&zBuf[4], 0);
+    put16(&zBuf[6], 0);
+    put16(&zBuf[8], nEntry);
+    put16(&zBuf[10], nEntry);
+    put32(&zBuf[12], iTocEnd - iTocStart);
+    put32(&zBuf[16], iTocStart);
+    put16(&zBuf[20], 0);
+    blob_append(&body, zBuf, 22);
+    blob_reset(&toc);
+    *(p->pBlob) = body;
+    blob_zero(&body);
+  }else{
+    if( p->db ) sqlite3_exec(p->db, "COMMIT", 0, 0, 0);
+    free_archive(p);
+    blob_reset(&p->tmp);
+  }
 
-  memset(zBuf, 0, sizeof(zBuf));
-  put32(&zBuf[0], 0x06054b50);
-  put16(&zBuf[4], 0);
-  put16(&zBuf[6], 0);
-  put16(&zBuf[8], nEntry);
-  put16(&zBuf[10], nEntry);
-  put32(&zBuf[12], iTocEnd - iTocStart);
-  put32(&zBuf[16], iTocStart);
-  put16(&zBuf[20], 0);
-  blob_append(&body, zBuf, 22);
-  blob_reset(&toc);
-  *pZip = body;
-  blob_zero(&body);
   nEntry = 0;
   for(i=0; i<nDir; i++){
     fossil_free(azDir[i]);
@@ -291,17 +579,25 @@ void filezip_cmd(void){
   int i;
   Blob zip;
   Blob file;
+  int eFType = SymFILE;
+  Archive sArchive;
+  memset(&sArchive, 0, sizeof(Archive));
+  sArchive.eType = ARCHIVE_ZIP;
+  sArchive.pBlob = &zip;
   if( g.argc<3 ){
     usage("ARCHIVE FILE....");
   }
+  if( find_option("dereference","h",0)!=0 ){
+    eFType = ExtFILE;
+  }
   zip_open();
   for(i=3; i<g.argc; i++){
     blob_zero(&file);
-    blob_read_from_file(&file, g.argv[i]);
-    zip_add_file(g.argv[i], &file, file_wd_perm(g.argv[i]));
+    blob_read_from_file(&file, g.argv[i], eFType);
+    zip_add_file(&sArchive, g.argv[i], &file, file_perm(0,0));
     blob_reset(&file);
   }
-  zip_close(&zip);
+  zip_close(&sArchive);
   blob_write_to_file(&zip, g.argv[2]);
 }
 
@@ -323,10 +619,11 @@ void filezip_cmd(void){
 ** with source files. For example, pass a UUID or "ProjectName".
 **
 */
-void zip_of_checkin(
-  int rid,            /* The RID of the checkin to construct the ZIP archive from */
-  Blob *pZip,         /* Write the ZIP archive content into this blob */
-  const char *zDir,   /* Top-level directory of the ZIP archive */
+static void zip_of_checkin(
+  int eType,          /* Type of archive (ZIP or SQLAR) */
+  int rid,            /* The RID of the checkin to build the archive from */
+  Blob *pZip,         /* Write the archive content into this blob */
+  const char *zDir,   /* Top-level directory of the archive */
   Glob *pInclude,     /* Only include files that match this pattern */
   Glob *pExclude      /* Exclude files that match this pattern */
 ){
@@ -336,9 +633,15 @@ void zip_of_checkin(
   Blob filename;
   int nPrefix;
 
+  Archive sArchive;
+  memset(&sArchive, 0, sizeof(Archive));
+  sArchive.eType = eType;
+  sArchive.pBlob = pZip;
+  blob_zero(&sArchive.tmp);
+  blob_zero(pZip);
+
   content_get(rid, &mfile);
   if( blob_size(&mfile)==0 ){
-    blob_zero(pZip);
     return;
   }
   blob_set_dynamic(&hash, rid_to_uuid(rid));
@@ -374,25 +677,20 @@ void zip_of_checkin(
         eflg |= MFESTFLG_TAGS;
       }
 
-      if( eflg & (MFESTFLG_RAW|MFESTFLG_UUID) ){
-        if( eflg & MFESTFLG_RAW ){
-          blob_append(&filename, "manifest", -1);
-          zName = blob_str(&filename);
-          zip_add_folders(zName);
-        }
-        if( eflg & MFESTFLG_RAW ){
-          sterilize_manifest(&mfile);
-          zip_add_file(zName, &mfile, 0);
-        }
+      if( eflg & MFESTFLG_RAW ){
+        blob_append(&filename, "manifest", -1);
+        zName = blob_str(&filename);
+        zip_add_folders(&sArchive, zName);
+        sterilize_manifest(&mfile);
+        zip_add_file(&sArchive, zName, &mfile, 0);
       }
-      blob_reset(&mfile);
       if( eflg & MFESTFLG_UUID ){
         blob_append(&hash, "\n", 1);
         blob_resize(&filename, nPrefix);
         blob_append(&filename, "manifest.uuid", -1);
         zName = blob_str(&filename);
-        zip_add_folders(zName);
-        zip_add_file(zName, &hash, 0);
+        zip_add_folders(&sArchive, zName);
+        zip_add_file(&sArchive, zName, &hash, 0);
       }
       if( eflg & MFESTFLG_TAGS ){
         Blob tagslist;
@@ -401,12 +699,13 @@ void zip_of_checkin(
         blob_resize(&filename, nPrefix);
         blob_append(&filename, "manifest.tags", -1);
         zName = blob_str(&filename);
-        zip_add_folders(zName);
-        zip_add_file(zName, &tagslist, 0);
+        zip_add_folders(&sArchive, zName);
+        zip_add_file(&sArchive, zName, &tagslist, 0);
         blob_reset(&tagslist);
       }
     }
     manifest_file_rewind(pManifest);
+    zip_add_file(&sArchive, "", 0, 0);
     while( (pFile = manifest_file_next(pManifest,0))!=0 ){
       int fid;
       if( pInclude!=0 && !glob_match(pInclude, pFile->zName) ) continue;
@@ -417,18 +716,67 @@ void zip_of_checkin(
         blob_resize(&filename, nPrefix);
         blob_append(&filename, pFile->zName, -1);
         zName = blob_str(&filename);
-        zip_add_folders(zName);
-        zip_add_file(zName, &file, manifest_file_mperm(pFile));
+        zip_add_folders(&sArchive, zName);
+        zip_add_file(&sArchive, zName, &file, manifest_file_mperm(pFile));
         blob_reset(&file);
       }
     }
-  }else{
-    blob_reset(&mfile);
   }
+  blob_reset(&mfile);
   manifest_destroy(pManifest);
   blob_reset(&filename);
   blob_reset(&hash);
-  zip_close(pZip);
+  zip_close(&sArchive);
+}
+
+/*
+** Implementation of zip_cmd and sqlar_cmd.
+*/
+static void archive_cmd(int eType){
+  int rid;
+  Blob zip;
+  const char *zName;
+  Glob *pInclude = 0;
+  Glob *pExclude = 0;
+  const char *zInclude;
+  const char *zExclude;
+
+  zName = find_option("name", 0, 1);
+  zExclude = find_option("exclude", "X", 1);
+  if( zExclude ) pExclude = glob_create(zExclude);
+  zInclude = find_option("include", 0, 1);
+  if( zInclude ) pInclude = glob_create(zInclude);
+  db_find_and_open_repository(0, 0);
+
+  /* We should be done with options.. */
+  verify_all_options();
+
+  if( g.argc!=4 ){
+    usage("VERSION OUTPUTFILE");
+  }
+  g.zOpenRevision = g.argv[2];
+  rid = name_to_typed_rid(g.argv[2], "ci");
+  if( rid==0 ){
+    fossil_fatal("Check-in not found: %s", g.argv[2]);
+    return;
+  }
+
+  if( zName==0 ){
+    zName = db_text("default-name",
+       "SELECT replace(%Q,' ','_') "
+          " || strftime('_%%Y-%%m-%%d_%%H%%M%%S_', event.mtime) "
+          " || substr(blob.uuid, 1, 10)"
+       "  FROM event, blob"
+       " WHERE event.objid=%d"
+       "   AND blob.rid=%d",
+       db_get("project-name", "unnamed"), rid, rid
+    );
+  }
+  zip_of_checkin(eType, rid, &zip, zName, pInclude, pExclude);
+  glob_free(pInclude);
+  glob_free(pExclude);
+  blob_write_to_file(&zip, g.argv[3]);
+  blob_reset(&zip);
 }
 
 /*
@@ -454,60 +802,45 @@ void zip_of_checkin(
 **   -R REPOSITORY           Specify a Fossil repository
 */
 void zip_cmd(void){
-  int rid;
-  Blob zip;
-  const char *zName;
-  Glob *pInclude = 0;
-  Glob *pExclude = 0;
-  const char *zInclude;
-  const char *zExclude;
-  zName = find_option("name", 0, 1);
-  zExclude = find_option("exclude", "X", 1);
-  if( zExclude ) pExclude = glob_create(zExclude);
-  zInclude = find_option("include", 0, 1);
-  if( zInclude ) pInclude = glob_create(zInclude);
-  db_find_and_open_repository(0, 0);
-
-  /* We should be done with options.. */
-  verify_all_options();
-
-  if( g.argc!=4 ){
-    usage("VERSION OUTPUTFILE");
-  }
-  rid = name_to_typed_rid(g.argv[2], "ci");
-  if( rid==0 ){
-    fossil_fatal("Check-in not found: %s", g.argv[2]);
-    return;
-  }
-
-  if( zName==0 ){
-    zName = db_text("default-name",
-       "SELECT replace(%Q,' ','_') "
-          " || strftime('_%%Y-%%m-%%d_%%H%%M%%S_', event.mtime) "
-          " || substr(blob.uuid, 1, 10)"
-       "  FROM event, blob"
-       " WHERE event.objid=%d"
-       "   AND blob.rid=%d",
-       db_get("project-name", "unnamed"), rid, rid
-    );
-  }
-  zip_of_checkin(rid, &zip, zName, pInclude, pExclude);
-  glob_free(pInclude);
-  glob_free(pExclude);
-  blob_write_to_file(&zip, g.argv[3]);
-  blob_reset(&zip);
+  archive_cmd(ARCHIVE_ZIP);
 }
 
 /*
-** WEBPAGE: zip
-** URL: /zip
+** COMMAND: sqlar*
 **
-** Generate a ZIP archive for the check-in specified by the "r"
-** query parameter.  Return that ZIP archive as the HTTP reply content.
+** Usage: %fossil sqlar VERSION OUTPUTFILE [OPTIONS]
+**
+** Generate an SQLAR archive for a check-in.  If the --name option is
+** used, its argument becomes the name of the top-level directory in the
+** resulting SQLAR archive.  If --name is omitted, the top-level directory
+** name is derived from the project name, the check-in date and time, and
+** the artifact ID of the check-in.
+**
+** The GLOBLIST argument to --exclude and --include can be a comma-separated
+** list of glob patterns, where each glob pattern may optionally be enclosed
+** in "..." or '...' so that it may contain commas.  If a file matches both
+** --include and --exclude then it is excluded.
+**
+** Options:
+**   -X|--exclude GLOBLIST   Comma-separated list of GLOBs of files to exclude
+**   --include GLOBLIST      Comma-separated list of GLOBs of files to include
+**   --name DIRECTORYNAME    The name of the top-level directory in the archive
+**   -R REPOSITORY           Specify a Fossil repository
+*/
+void sqlar_cmd(void){
+  archive_cmd(ARCHIVE_SQLAR);
+}
+
+/*
+** WEBPAGE: sqlar
+** WEBPAGE: zip
+**
+** Generate a ZIP or SQL archive for the check-in specified by the "r"
+** query parameter.  Return the archive as the HTTP reply content.
 **
 ** Query parameters:
 **
-**   name=NAME[.zip]     The base name of the output file.  The default
+**   name=NAME           The base name of the output file.  The default
 **                       value is a configuration parameter in the project
 **                       settings.  A prefix of the name, omitting the
 **                       extension, is used as the top-most directory name.
@@ -516,7 +849,7 @@ void zip_cmd(void){
 **                       Defaults to "trunk".  This query parameter used to
 **                       be called "uuid" and the older "uuid" name is still
 **                       accepted for backwards compatibility.  If this
-**                       query paramater is omitted, the latest "trunk"
+**                       query parameter is omitted, the latest "trunk"
 **                       check-in is used.
 **
 **   in=PATTERN          Only include files that match the comma-separate
@@ -538,27 +871,46 @@ void baseline_zip_page(void){
   Glob *pInclude = 0;           /* The compiled in= glob pattern */
   Glob *pExclude = 0;           /* The compiled ex= glob pattern */
   Blob zip;                     /* ZIP archive accumulated here */
+  int eType = ARCHIVE_ZIP;      /* Type of archive to generate */
+  char *zType;                  /* Human-readable archive type */
 
   login_check_credentials();
   if( !g.perm.Zip ){ login_needed(g.anon.Zip); return; }
+  if( fossil_strcmp(g.zPath, "sqlar")==0 ){
+    eType = ARCHIVE_SQLAR;
+    zType = "SQL";
+  }else{
+    eType = ARCHIVE_ZIP;
+    zType = "ZIP";
+  }
   load_control();
   zName = mprintf("%s", PD("name",""));
   nName = strlen(zName);
   z = P("r");
   if( z==0 ) z = P("uuid");
   if( z==0 ) z = "trunk";
-  zRid = fossil_strdup(z);
+  g.zOpenRevision = zRid = fossil_strdup(z);
   nRid = strlen(zRid);
   zInclude = P("in");
   if( zInclude ) pInclude = glob_create(zInclude);
   zExclude = P("ex");
   if( zExclude ) pExclude = glob_create(zExclude);
-  if( nName>4 && fossil_strcmp(&zName[nName-4], ".zip")==0 ){
+  if( eType==ARCHIVE_ZIP 
+   && nName>4
+   && fossil_strcmp(&zName[nName-4], ".zip")==0
+  ){
     /* Special case:  Remove the ".zip" suffix.  */
     nName -= 4;
     zName[nName] = 0;
+  }else if( eType==ARCHIVE_SQLAR 
+   && nName>6
+   && fossil_strcmp(&zName[nName-6], ".sqlar")==0
+  ){
+    /* Special case:  Remove the ".sqlar" suffix.  */
+    nName -= 6;
+    zName[nName] = 0;
   }else{
-    /* If the file suffix is not ".zip" then just remove the
+    /* If the file suffix is not ".zip" or ".sqlar" then just remove the
     ** suffix up to and including the last "." */
     for(nName=strlen(zName)-1; nName>5; nName--){
       if( zName[nName]=='.' ){
@@ -567,8 +919,9 @@ void baseline_zip_page(void){
       }
     }
   }
-  rid = name_to_typed_rid(nRid?zRid:zName, "ci");
-  if( rid==0 ){
+  rid = symbolic_name_to_rid(nRid?zRid:zName, "ci");
+  if( rid<=0 ){
+    cgi_set_status(404, "Not Found");
     @ Not found
     return;
   }
@@ -576,14 +929,14 @@ void baseline_zip_page(void){
 
   /* Compute a unique key for the cache entry based on query parameters */
   blob_init(&cacheKey, 0, 0);
-  blob_appendf(&cacheKey, "/zip/%z", rid_to_uuid(rid));
+  blob_appendf(&cacheKey, "/%s/%z", g.zPath, rid_to_uuid(rid));
   blob_appendf(&cacheKey, "/%q", zName);
   if( zInclude ) blob_appendf(&cacheKey, ",in=%Q", zInclude);
   if( zExclude ) blob_appendf(&cacheKey, ",ex=%Q", zExclude);
   zKey = blob_str(&cacheKey);
 
   if( P("debug")!=0 ){
-    style_header("ZIP Archive Generator Debug Screen");
+    style_header("%s Archive Generator Debug Screen", zType);
     @ zName = "%h(zName)"<br />
     @ rid = %d(rid)<br />
     if( zInclude ){
@@ -597,11 +950,11 @@ void baseline_zip_page(void){
     return;
   }
   if( referred_from_login() ){
-    style_header("ZIP Archive Download");
-    @ <form action='%R/zip/%h(zName).zip'>
+    style_header("%s Archive Download", zType);
+    @ <form action='%R/%s(g.zPath)/%h(zName).%s(g.zPath)'>
     cgi_query_parameters_to_hidden();
-    @ <p>ZIP Archive named <b>%h(zName).zip</b> holding the content
-    @ of check-in <b>%h(zRid)</b>:
+    @ <p>%s(zType) Archive named <b>%h(zName).%s(g.zPath)</b>
+    @ holding the content of check-in <b>%h(zRid)</b>:
     @ <input type="submit" value="Download" />
     @ </form>
     style_footer();
@@ -609,7 +962,7 @@ void baseline_zip_page(void){
   }
   blob_zero(&zip);
   if( cache_read(&zip, zKey)==0 ){
-    zip_of_checkin(rid, &zip, zName, pInclude, pExclude);
+    zip_of_checkin(eType, rid, &zip, zName, pInclude, pExclude);
     cache_write(&zip, zKey);
   }
   glob_free(pInclude);
@@ -618,5 +971,9 @@ void baseline_zip_page(void){
   fossil_free(zRid);
   blob_reset(&cacheKey);
   cgi_set_content(&zip);
-  cgi_set_content_type("application/zip");
+  if( eType==ARCHIVE_ZIP ){
+    cgi_set_content_type("application/zip");
+  }else{
+    cgi_set_content_type("application/sqlar");
+  }
 }
