@@ -38,6 +38,7 @@ set doclist {
   foss-cklist.wiki {Checklist For Successful Open-Source Projects}
   fossil-from-msvc.wiki {Integrating Fossil in the Microsoft Express 2010 IDE}
   fossil-v-git.wiki {Fossil Versus Git}
+  globs.md {File Name Glob Patterns}
   hacker-howto.wiki {Hacker How-To}
   hashpolicy.wiki {Hash Policy: Choosing Between SHA1 and SHA3-256}
   /help {Lists of Commands and Webpages}
@@ -119,7 +120,7 @@ puts $out {
 <li> <a href='../COPYRIGHT-BSD2.txt'>License</a>
 <li> <a href='http://www.fossil-scm.org/schimpf-book/home'>Jim Schimpf's
 book</a>
-<li> <a href='$ROOT/help'>Command-line help</a>
+<li> <a href='$ROOT/help'>List of commands, web-pages, and settings</a>
 </ul>
 <a name="pindex"></a>
 <h2>Permuted Index:</h2>
